/*
 *
 * Mandrake KWin client configuration module based on Keramik kwin client
 * Copyright (C) 2003 Laurent Montel <lmontel@mandrakesoft.com>
 * Copyright (C) 2002 Fredrik Hglund <fredrik@kde.org>
 *
 * Based on the Quartz configuration module,
 *     Copyright (c) 2001 Karol Szwed <gallium@kde.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the license, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <kglobal.h>
#include <klocale.h>

#include <qcheckbox.h>

#include "config.h"
#include "config.moc"

extern "C"
{
	QObject* allocate_config( KConfig* conf, QWidget* parent )
	{
		return ( new MandrakeConfig( conf, parent ) );
	}
}


/* NOTE:
 * 'conf' 	is a pointer to the kwindecoration modules open kwin config,
 *			and is by default set to the "Style" group.
 *
 * 'parent'	is the parent of the QObject, which is a VBox inside the
 *			Configure tab in kwindecoration
 */

MandrakeConfig::MandrakeConfig( KConfig* conf, QWidget* parent )
	: QObject( parent )
{
	KGlobal::locale()->insertCatalogue("kwin_mandrake_config");
	c = new KConfig( "kwinmandrakerc" );

	ui = new MandrakeConfigUI( parent );
	connect( ui->useShadowedText, SIGNAL(clicked()), SIGNAL(changed()) );

	load( conf );
	ui->show();
}


MandrakeConfig::~MandrakeConfig()
{
	delete ui;
	delete c;
}


// Loads the configurable options from the kwinrc config file
// It is passed the open config from kwindecoration to improve efficiency
void MandrakeConfig::load( KConfig* )
{
	c->setGroup("General");
	ui->useShadowedText->setChecked( c->readBoolEntry("UseShadowedText", true) );
}


// Saves the configurable options to the kwinrc config file
void MandrakeConfig::save( KConfig* )
{
	c->setGroup( "General" );
	c->writeEntry( "UseShadowedText", ui->useShadowedText->isChecked() );
	c->sync();
}


// Sets UI widget defaults which must correspond to style defaults
void MandrakeConfig::defaults()
{
	ui->useShadowedText->setChecked( true );

	emit changed();
}
