/*
 * Copyright (C) 2003 Laurent Montel <lmontel@mandrakesoft.com>
 *
 * based on KDE3 HighColor Style (version 1.0)
 * Copyright (C) 2001-2002 Karol Szwed      <gallium@kde.org>
 *           (C) 2001-2002 Fredrik Hglund  <fredrik@kde.org>
 *
 * Drawing routines adapted from the KDE2 HCStyle,
 * Copyright (C) 2000 Daniel M. Duley       <mosfet@kde.org>
 *           (C) 2000 Dirk Mueller          <mueller@kde.org>
 *           (C) 2001 Martijn Klingens      <klingens@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <qdict.h>
#include <qdrawutil.h>
#include <qpainter.h>
#include <qpointarray.h>
#include <qstyleplugin.h>
#include <qprogressbar.h>

#include <qcombobox.h>
#include <qheader.h>
#include <qmenubar.h>
#include <qpushbutton.h>
#include <qscrollbar.h>
#include <qslider.h>
#include <qtabbar.h>
#include <qtoolbutton.h>
#include <qtoolbar.h>
#include <qpopupmenu.h>

#include <kdrawutil.h>
#include <kpixmapeffect.h>
#include <qradiobutton.h>
#include <qimage.h>

#include "thememdk.h"
#include "thememdk.moc"
#include "bitmaps.h"
#include <qlistview.h>


// -- Style Plugin Interface -------------------------
class ThemeMdkStylePlugin : public QStylePlugin
{
public:
  ThemeMdkStylePlugin() {}
  ~ThemeMdkStylePlugin() {}

  QStringList keys() const
  {
    return QStringList() << "Galaxy";
  }

  QStyle* create( const QString& key )
  {
      if ( key == "galaxy" )
          return new MandrakeStyle( );
      return 0;
  }
};

Q_EXPORT_PLUGIN( ThemeMdkStylePlugin )
// ---------------------------------------------------


// ### Remove globals
QBitmap lightBmp;
QBitmap grayBmp;
QBitmap dgrayBmp;
QBitmap centerBmp;
QBitmap maskBmp;
QBitmap xBmp;
QBitmap menuBmp;
QDict<GradientSet> gDict;

static bool scrollmin = false;
static bool scrollmax = false;


static const int itemFrame       = 1;
static const int itemHMargin     = 3;
static const int itemVMargin     = 0;
static const int arrowHMargin    = 6;
static const int rightBorder     = 12;
static const char* kdeToolbarWidget = "kde toolbar widget";
const int smallButMaxW    = 27;
const int smallButMaxH    = 20;

// QColor("#CECECE") == cg.background().dark(112)
// QColor("#EFEFEF") == cg.background().dark(97)
// QColor("#A5A6A5") == cg.background().dark(139)
// QColor("#CECFCE") == cg.background().dark(112) == cdata->shades[2]
// QColor("#adaaad") == cg.background().dark(132)
// ---------------------------------------------------------------------------
GradientSet::GradientSet(const QColor &_begin, const QColor &_end)
{
    begin=_begin;
    end=_end;
    for(int i=0; i < GradientCount; i++)
        gradients[i] = NULL;
}


GradientSet::~GradientSet()
{
    for(int i=0; i < GradientCount; i++)
        delete gradients[i];
}


KPixmap* GradientSet::gradient(GradientType type)
{
    if (gradients[type])
        return gradients[type];

    switch(type)
    {
    case VSmall: {
        gradients[VSmall] = new KPixmap;
        gradients[VSmall]->resize(18, 24);
        KPixmapEffect::gradient(*gradients[VSmall], begin, end,KPixmapEffect::VerticalGradient);
        break;
    }

    case VMed: {
        gradients[VMed] = new KPixmap;
        gradients[VMed]->resize(18, 34);
        KPixmapEffect::gradient(*gradients[VMed], begin, end,
                                KPixmapEffect::VerticalGradient);
        break;
    }

    case VLarge: {
        gradients[VLarge] = new KPixmap;
        gradients[VLarge]->resize(18, 64);
        KPixmapEffect::gradient(*gradients[VLarge], begin, end,
                                KPixmapEffect::VerticalGradient);
        break;
    }
    case HMed: {
        gradients[HMed] = new KPixmap;
        gradients[HMed]->resize(34, 18);
        KPixmapEffect::gradient(*gradients[HMed], begin, end,
                                KPixmapEffect::HorizontalGradient);
        break;
    }

    case HLarge: {
        gradients[HLarge] = new KPixmap;
        gradients[HLarge]->resize(52, 18);
        KPixmapEffect::gradient(*gradients[HLarge], begin, end,
                                KPixmapEffect::HorizontalGradient);
        break;
    }

    default:
        break;
    }
    return(gradients[type]);
}



#define RADIO_SIZE 13

const double MandrakeStyle::shading[8] = {1.036875969, 1.0 , 0.893708749, 0.822125139,
                      0.74403433, 0.71800443, 0.646420819,0.587006644};



static void rgb_to_hls (double *r,
            double *g,
            double *b)
{
  double min;
  double max;
  double red;
  double green;
  double blue;
  double h, l, s;
  double delta;

  red = *r;
  green = *g;
  blue = *b;

  if (red > green) {
    if (red > blue)
      max = red;
    else
      max = blue;

    if (green < blue)
      min = green;
    else
      min = blue;
  } else {
    if (green > blue)
      max = green;
    else
      max = blue;

    if (red < blue)
      min = red;
    else
      min = blue;
  }

  l = (max + min) / 2;
  s = 0;
  h = 0;

  if (max != min) {
    if (l <= 0.5)
      s = (max - min) / (max + min);
    else
      s = (max - min) / (2 - max - min);

    delta = max -min;
    if (red == max)
      h = (green - blue) / delta;
    else if (green == max)
      h = 2 + (blue - red) / delta;
    else if (blue == max)
      h = 4 + (red - green) / delta;

    h *= 60;
    if (h < 0.0)
      h += 360;
  }

  *r = h;
  *g = l;
  *b = s;
}

static void hls_to_rgb (double *h,
            double *l,
            double *s)
{
  double hue;
  double lightness;
  double saturation;
  double m1, m2;
  double r, g, b;

  lightness = *l;
  saturation = *s;

  if (lightness <= 0.5)
    m2 = lightness * (1 + saturation);
  else
    m2 = lightness + saturation - lightness * saturation;
  m1 = 2 * lightness - m2;

  if (saturation == 0) {
    *h = lightness;
    *l = lightness;
    *s = lightness;
  } else {
    hue = *h + 120;
    while (hue > 360)
      hue -= 360;
    while (hue < 0)
      hue += 360;

    if (hue < 60)
      r = m1 + (m2 - m1) * hue / 60;
    else if (hue < 180)
      r = m2;
    else if (hue < 240)
      r = m1 + (m2 - m1) * (240 - hue) / 60;
    else
      r = m1;

    hue = *h;
    while (hue > 360)
      hue -= 360;
    while (hue < 0)
      hue += 360;

    if (hue < 60)
      g = m1 + (m2 - m1) * hue / 60;
    else if (hue < 180)
      g = m2;
    else if (hue < 240)
      g = m1 + (m2 - m1) * (240 - hue) / 60;
    else
      g = m1;

    hue = *h - 120;
    while (hue > 360)
      hue -= 360;
    while (hue < 0)
      hue += 360;

    if (hue < 60)
      b = m1 + (m2 - m1) * hue / 60;
    else if (hue < 180)
      b = m2;
    else if (hue < 240)
      b = m1 + (m2 - m1) * (240 - hue) / 60;
    else
      b = m1;

    *h = r;
    *l = g;
    *s = b;
  }
}

static void shade (const QColor &ca, QColor &cb, double k)
{
  int r,g,b;
  double red;
  double green;
  double blue;

  ca.rgb (&r, &g, &b);

  red = (double) r / 255.0;
  green = (double) g / 255.0;
  blue = (double) b / 255.0;

  rgb_to_hls (&red, &green, &blue);

  green *= k;
  if (green > 1.0)
    green = 1.0;
  else if (green < 0.0)
    green = 0.0;

  blue *= k;
  if (blue > 1.0)
    blue = 1.0;
  else if (blue < 0.0)
    blue = 0.0;

  hls_to_rgb (&red, &green, &blue);

  cb.setRgb ((int)(red*255.0), (int)(green*255.0), (int)(blue*255.0));
}

static QImage *
generate_bit (unsigned char alpha[], const QColor &color, double mult)
{
  unsigned int r, g, b;
  QImage *image;
  QRgb *pixels;
  int w, h;
  int x, y;

  r = (int) (color.red() * mult);
  r = QMIN(r, 255);
  g = (int) (color.green() * mult);
  g = QMIN(g, 255);
  b = (int) (color.blue() * mult);
  b = QMIN(b, 255);

  image = new QImage (RADIO_SIZE, RADIO_SIZE, 32);
  image->setAlphaBuffer(true);

  w = image->width();
  h = image->height();

  for (y=0; y < h; y++)
    {
      pixels = (QRgb *)image->scanLine(y);
      for (x=0; x < w; x++)
        pixels[x] = qRgba (r, g, b, alpha?alpha[y*w+x]:255);
    }

  return image;
}

#define CLAMP(x, low, high)  (((x) > (high)) ? (high) : (((x) < (low)) ? (low) : (x)))
#define CLAMP_UCHAR(v) ((unsigned char) (CLAMP (((int)v), (int)0, (int)255)))

static QImage *
colorize_bit (unsigned char *bit,
              unsigned char *alpha,
              const QColor  &new_color)
{
  QImage *image;
  double intensity;
  int x, y;
  const unsigned char *src, *asrc;
  QRgb *dest;
  int dest_rowstride;
  int width, height;
  unsigned char *dest_pixels;

  image = new QImage (RADIO_SIZE, RADIO_SIZE, 32, 0, QImage::BigEndian);
  image->setAlphaBuffer(true);

  if (image == NULL)
    return NULL;

  dest_rowstride = image->bytesPerLine();
  width = image->width();
  height = image->height();
  dest_pixels = image->bits();

  for (y = 0; y < RADIO_SIZE; y++)
    {
      src = bit + y * RADIO_SIZE;
      asrc = alpha + y * RADIO_SIZE;
      dest = (QRgb *)image->scanLine (y);

      for (x = 0; x < RADIO_SIZE; x++)
        {
          int dr, dg, db;

          intensity = src[x]/ 255.0;

          if (intensity <= 0.5)
            {
              /* Go from black at intensity = 0.0 to new_color at intensity = 0.5 */
              dr = int((new_color.red() * intensity * 2.0));
              dg = int((new_color.green() * intensity * 2.0));
              db = int((new_color.blue() * intensity * 2.0));
            }
          else
            {
              /* Go from new_color at intensity = 0.5 to white at intensity = 1.0 */
              dr = int((new_color.red() + (255 - new_color.red()) * (intensity - 0.5) * 2.0));
              dg = int((new_color.green() + (255 - new_color.green()) * (intensity - 0.5) * 2.0));
              db = int((new_color.blue() + (255 - new_color.blue()) * (intensity - 0.5) * 2.0));
            }

          dest[x] = qRgba (CLAMP_UCHAR (dr), CLAMP_UCHAR (dg), CLAMP_UCHAR (db), asrc[x]);
        }
    }

  return image;
}


static void composeImage (QImage *destImg, QImage *srcImg)
{
  int w, h, x, y;
  QRgb *src, *dest;
  unsigned int a;
  QRgb s, d;

  w = destImg->width();
  h = destImg->height();

  for (y = 0; y < h; y++)
    {
      src = (QRgb *)srcImg->scanLine(y);
      dest = (QRgb *)destImg->scanLine(y);

      for (x = 0; x < w; x++)
        {
          s = src[x];
          d = dest[x];

          a = qAlpha(s);

          dest[x] = qRgba ((qRed(s) * a + (255-a)*qRed(d)) / 255,
                           (qGreen(s) * a + (255-a)*qGreen(d)) / 255,
                           (qBlue(s) * a + (255-a)*qBlue(d)) / 255,
                           a + ((255-a)*qAlpha(d)) / 255);
        }
    }
}


MandrakeStyle::MandrakeColorData::~MandrakeColorData()
{
  int i;

  for (i = 0; i < 8; i++) {
      delete radioPix[i];
  }
  delete radioMask;
}


// ---------------------------------------------------------------------------

MandrakeStyle::MandrakeStyle( )
    : KStyle( AllowMenuTransparency | FilledFrameWorkaround, WindowsStyleScrollBar )
      , m_dataCache ()
{
    gDict.setAutoDelete(true);
    hoverWidget = 0L;
    m_dataCache.setAutoDelete(true);

}


MandrakeStyle::~MandrakeStyle()
{
}

MandrakeStyle::MandrakeColorData *MandrakeStyle::realizeData (const QColorGroup &cg) const
{
  MandrakeColorData *cdata;
  int i, j;

  cdata = new MandrakeColorData;
  cdata->buttonColor = cg.button().rgb();
  cdata->spotColor = cg.button().rgb();

  //initialise grey color
  for (i = 0; i < 8; i++) {
    shade (cg.background(), cdata->shades[i], shading[i]);
  }
  //blue mdk color
  shade (cg.highlight(), cdata->blueMdkColor[0], 1.0);//21449c
  shade (cg.highlight(), cdata->blueMdkColor[1], 1.204945256);//intermedial blue
  if( cg.highlight() == QColor("#21449C"))
     cdata->blueMdkColor[2]=QColor("#7382BD");
  else
     shade (cg.highlight(), cdata->blueMdkColor[2], 1.204945256);//intermedial blue

  shade (cg.background(), cdata->radioColor, 0.2 );

  QImage *dot, *outline, *circle;

  dot = colorize_bit (dot_intensity, dot_alpha, cdata->radioColor);
  outline = generate_bit (outline_alpha, cdata->shades[6], 1.0);

  QImage composite (RADIO_SIZE, RADIO_SIZE, 32, 0, QImage::BigEndian);

  for (i = 0; i < 2; i++) {
    for (j = 0; j < 2; j++) {
      if (i == 0)
	composite.fill (/*cdata->shades[0]*/cg.background().rgb());
      else
	composite.fill (/*cdata->shades[0]*/cg.background().rgb()/*FIXME ! */);
      composeImage (&composite, outline);

      //FIXME !!!!!!!!!!!!!!! bad color when we press .
      if (j == 0)
        circle = generate_bit (circle_alpha, Qt::white, 1.0);
      else
	circle = generate_bit (circle_alpha, cdata->shades[1] , 1.0);

      composeImage (&composite, circle);
      delete circle;

      cdata->radioPix[i*4+j*2+0] = new QPixmap (composite);

      composeImage (&composite, dot);
      cdata->radioPix[i*4+j*2+1] = new QPixmap (composite);
    }
  }

  QImage mask = outline->createAlphaMask();
  cdata->radioMask = new QPixmap (mask);

  delete dot;
  delete outline;
  return cdata;
}

/* We assume this seldom collides, since we can only cache one at a time */
static long hashColorGroup (const QColorGroup &cg)
{
    return cg.button().rgb();// << 8 ^ cg.button().rgb();
}


const MandrakeStyle::MandrakeColorData *MandrakeStyle::lookupData (const QColorGroup &cg) const
{
  MandrakeColorData *cdata;
  long h;
  QIntCache<MandrakeColorData> *cache;

  h = hashColorGroup (cg);

  // Cast away the const crap
  // Nasty, but how else would i do it?
  cache = (QIntCache<MandrakeColorData> *)&m_dataCache;

  cdata = cache->find(h);

  if (cdata == 0 || !cdata->isGroup(cg)) {
    if (cdata != 0) {
      cache->remove (h);
    }

    cdata = realizeData (cg);
    cache->insert (h, cdata);
  }

  return cdata;
}


void MandrakeStyle::getShade (const QColorGroup &cg, int shadenr, QColor &res) const
{
  const MandrakeColorData *cdata = lookupData(cg);

  res = cdata->shades[shadenr];
}



void MandrakeStyle::polish(QWidget* widget)
{
	// Put in order of highest occurance to maximise hit rate
  if (widget->inherits("QPushButton") || widget->inherits( "QComboBox" ) || widget->inherits("QScrollBar") || widget->inherits("QHeader")) {
		widget->installEventFilter(this);
	} else if (widget->inherits("QMenuBar") || widget->inherits("QPopupMenu")) {
		widget->setBackgroundMode(QWidget::NoBackground);
	}
        else if (widget->inherits("QToolBarExtensionWidget")) {
		widget->installEventFilter(this);
	}
        else if ( !qstrcmp( widget->name(), kdeToolbarWidget) ) {
		widget->setBackgroundMode( NoBackground );	// We paint the whole background.
		widget->installEventFilter(this);
	}
	KStyle::polish( widget );
}


void MandrakeStyle::unPolish(QWidget* widget)
{
	if (widget->inherits("QPushButton")|| widget->inherits( "QComboBox"  )|| widget->inherits("QScrollBar")|| widget->inherits("QHeader")) {
		widget->removeEventFilter(this);
	}
	else if (widget->inherits("QMenuBar") || widget->inherits("QPopupMenu")) {
		widget->setBackgroundMode(QWidget::PaletteBackground);
	}
        else if (widget->inherits("QToolBarExtensionWidget")) {
		widget->removeEventFilter(this);
	}
        else if ( !qstrcmp( widget->name(), kdeToolbarWidget) ) {
		widget->removeEventFilter(this);
		widget->setBackgroundMode( PaletteBackground );
	}

	KStyle::unPolish( widget );
}


/* reimp. */
void MandrakeStyle::renderMenuBlendPixmap( KPixmap& pix, const QColorGroup &cg,
		const QPopupMenu* /* popup */ ) const
{
	QColor col = cg.button();
	//FIXME !!!!!!!!!!!!!!!
#ifdef Q_WS_X11 // Only draw menu gradients on TrueColor, X11 visuals
	if ( QPaintDevice::x11AppDepth() >= 24 )
		KPixmapEffect::gradient( pix, col.light(120), col.dark(115),
				KPixmapEffect::HorizontalGradient );
	else
#endif
	pix.fill( col );
}


void MandrakeStyle::drawHandle( const MandrakeColorData *cdata,
                                QPainter *p,
                                const QRect &r,
                                const QColorGroup & cg,
                                SFlags flags,
                                const QStyleOption& /*opt*/,
                                bool drawBorder, bool drawGradient) const
{
    int x,y,width,height;
    r.rect(&x, &y, &width, &height);
    int x2 = x+width;
    int y2 = y+height-1;

    int i = 0;
    int delta=0;
    if( drawBorder )
    {
        p->setPen(  cg.base());
        p->drawLine( x2,y,x2,y2);
        p->drawLine( x,y,x,y2);


        p->setPen(  cdata->shades[4]);
        p->drawLine( x2-1,y2,x2-1,y);
        p->setPen( cdata->shades[5]);
        p->drawLine( x-2,y2,x2-2,y2);
        p->drawLine( x2-2,y2,x2-2,y);
    }
    //fix for handle toolbar, otherwise it's not center
    if(drawBorder)
      x -= 1;
    if (!(flags & Style_Horizontal))
    {

        if ( drawGradient )
            renderGradient( p, QRect(x+2,y+2,width-4, height-4),
                            cdata->shades[2], cdata->shades[0],false);
        if(width > 30)
        {
            delta = x + (width/2) - 11;
            for(i = 0; i < 3; i++)
            {
                drawHandlePoint (cdata,p, (delta + 9*i),
                                 y + QMAX((height-4)/2,0));
            }
        }
        else
        {
            drawHandlePoint (cdata, p,  x + (width-4)/2 ,
                             y + QMAX((height-4)/2,0));
        }
    }
    else
    {
        if( drawGradient )
            renderGradient(p, QRect(x+2,y+2,width-4, height-4),
                           cdata->shades[0],cdata->shades[2], true);

        if(height > 30)
        {
            delta = y + (height / 2) - 11;
            for(i = 0; i < 3; i++)
            {
                drawHandlePoint (cdata, p, x + QMAX((width-4)/2,0),
                                 delta+9*i);
            }
        }
        else
        {
            drawHandlePoint (cdata,p, x + QMAX((width-4)/2,0),
                             y + (height-4) / 2);
        }
    }
}

void MandrakeStyle::drawFrame( const MandrakeColorData *cdata, const QColorGroup & cg, QPainter *p, const QRect & r, bool checkbox )const
{
    int x,y,w,h;
    x=r.x(); y=r.y(); w=r.width(); h=r.height();
    int x2 = x+w-1;
    int y2 = y+h-1;

    p->setPen(cdata->shades[2]/*CECECE*/);
    p->drawLine(x, y, checkbox ? x2 : x2-1, y);
    p->drawLine(x, y, x, checkbox ? y2 : y2-1);

    p->setPen(cg.base()/*white*/);
    p->drawLine(x2, checkbox ? y+1 : y, x2, y2);
    p->drawLine(checkbox ? x+1 : x, y2, x2, y2);

    p->setPen(cdata->shades[5]);
    p->drawLine(x+1, y+1, x2-1, y+1);
    p->drawLine(x+1, y+1, x+1, y2-1);

    p->setPen(cdata->shades[0]);
    p->drawLine(x2-1, y+2, x2-1, y2-1);
    p->drawLine(x+2, y2-1, x2-1, y2-1);
}
#include <kdebug.h>
// This function draws primitive elements as well as their masks.
void MandrakeStyle::drawPrimitive( PrimitiveElement pe,
                                   QPainter *p,
                                   const QRect &r,
                                   const QColorGroup &cg,
                                   SFlags flags,
                                   const QStyleOption& opt ) const
{
  int x, y, xw, yh;
  x= r.x();
  y= r.y();
  xw = r.x()+r.width()-1;
  yh = r.y()+r.height()-1;
  bool down = flags & Style_Down;
  bool on   = flags & Style_On;
  const MandrakeColorData *cdata = lookupData(cg);

  switch(pe)
    {
    case PE_FocusRect:
      {
	p->save();
	p->setPen(cdata->shades[8]);
        p->drawWinFocusRect( r );
	p->restore();
        break;
      }
    case PE_PanelMenuBar: 			// Menu
      {
	//FIXME
        p->fillRect(r, cg.brush(QColorGroup::Button));

	int x2 = r.x()+r.width()-1;
	int y2 = r.y()+r.height()-1;
	int lw = opt.isDefault() ? pixelMetric(PM_DefaultFrameWidth)
	  : opt.lineWidth();
	p->setPen(cdata->shades[7]);
	if (lw)
	  {
	    p->drawLine(x2, y, x2, y2);
	  }
	p->setPen(cdata->shades[7]);
	p->drawLine(x, y2, x2, y2);

	break;
      }


    case PE_PanelDockWindow:		// Toolbar
      {
	bool horiz = r.width() > r.height();

	//Here, we just draw the border.
	int x  = r.x();
	int y  = r.y();
	int x2 = r.x() + r.width()  - 1;
	int y2 = r.y() + r.height() - 1;
	int lw = opt.isDefault() ? pixelMetric(PM_DefaultFrameWidth)
	  : opt.lineWidth();

	//FIXME
	if (lw)
	  {
	    //Gradient border colors.
	    //(Same as in gradients.cpp)
	    QColor gradTop = cg.base();
	    QColor gradBot = cdata->shades[7/*2*/];
	    if (horiz)
	      {
		//Top line
		p->setPen(gradTop);
		p->drawLine(x, y, x2, y);

		//Bottom line
		p->setPen(gradBot);
		p->drawLine(x, y2, x2, y2);

		//Left line

		//Right end-line
		p->setPen(cdata->shades[7]);
		p->drawLine(x2, y, x2, y2);
	      }
	    else
	      {
		//Left line
		p->setPen(gradTop);
		p->drawLine(x, y, x, y2);

		//Right line
		p->setPen(gradBot);
		p->drawLine(x2, y, x2, y2);


		//Bottom end-line
		p->setPen(cdata->shades[7]);
		p->drawLine(x, y2, x2, y2);
	      }
	  }
	break;
      }
      //todo PE_DockWindowSeparator

    case PE_DockWindowResizeHandle:
    case PE_Splitter:
      {
	QRect real(r);
	real.addCoords( 1, 0,0,0);
        drawHandle( cdata, p,real,cg,flags,opt, false );
        break;
      }
    case PE_PanelLineEdit:
      {
        drawFrame( cdata, cg, p, r );
        break;
      }

      //TODO !!!!!!!!!!!!!!!!!!!! FIXME !!!!!!
    case PE_Panel:
    case PE_PanelPopup:
    case PE_WindowFrame:
      {
        bool sunken  = flags & Style_Sunken;
        int lw = /*opt.isDefault() ? pixelMetric(PM_DefaultFrameWidth)
                   : opt.lineWidth()*/1;
        if (lw == 1 || lw == 2)
	  {
            QPen oldPen = p->pen();
            int x,y,w,h;
            r.rect(&x, &y, &w, &h);
            int x2 = x+w-1;
            int y2 = y+h-1;
            p->setPen(sunken ? cg.light() : cdata->shades[2]);
            p->drawLine(x, y2, x2, y2);
            p->drawLine(x2, y, x2, y2);
            p->setPen(sunken ? cdata->shades[2] : cg.light());
            p->drawLine(x, y, x2, y);
            p->drawLine(x, y, x, y2);
            p->setPen(sunken ? cg.midlight() : cdata->shades[2]);
            p->drawLine(x+1, y2-1, x2-1, y2-1);
            p->drawLine(x2-1, y+1, x2-1, y2-1);
            p->setPen(sunken ? cdata->shades[2] : cg.midlight());
            p->drawLine(x+1, y+1, x2-1, y+1);
            p->drawLine(x+1, y+1, x+1, y2-1);
            if( lw == 2 )
	      {
		p->setPen(sunken ? cg.midlight() : cdata->shades[5]);
                p->drawLine(x+2, y2-2, x2-2, y2-2);
                p->drawLine(x2-1, y+2, x2-2, y2-2);
                p->setPen(sunken ? cdata->shades[5] : cg.midlight());
                p->drawLine(x+2, y+2, x2-2, y+2);
                p->drawLine(x+2, y+2, x+2, y2-2);
	      }
            p->setPen(oldPen);
	  }
        else
	  KStyle::drawPrimitive(pe, p, r, cg, flags, opt);
        break;
      }
#if 0
    case PE_PanelGroupBox:
      drawPrimitive( PE_GroupBoxFrame, p, r, cg, flags, opt );
      break;
    case PE_GroupBoxFrame:
      {
	if ( opt.isDefault() )
	  break;
	int lwidth = opt.lineWidth(), mlwidth = opt.midLineWidth();
	if ( flags & (Style_Sunken|Style_Raised))
	  {
            drawShadeRect( p, r.x(), r.y(), r.width(), r.height(), cg, flags & Style_Sunken, lwidth, mlwidth );
	  }
	else
	  {
	    drawPlainRect( p, r.x(), r.y(), r.width(), r.height(), /*cg.foreground()*/cdata->shades[1], lwidth );
	  }
        break;
      }
#endif
    case PE_Separator:
      {
	drawShadeLine( p, r.left(), r.top(), r.right(), r.bottom(), cg,
                       flags & Style_Sunken, 1, 0);
	break;
      }
    case PE_HeaderArrow:
      {
        if ( flags & Style_Up )
	  drawPrimitive( PE_ArrowUp, p, r, cg, Style_Enabled );
        else
	  drawPrimitive( PE_ArrowDown, p, r, cg, Style_Enabled );
        break;
      }
    case PE_HeaderSection:
      {
        // Temporary solution for the proper orientation of gradients.
        bool horizontal = true;
        if (p && p->device()->devType() == QInternal::Widget) {
	  QHeader* hdr = dynamic_cast<QHeader*>(p->device());
	  if (hdr)
	    horizontal = hdr->orientation() == Horizontal;
        }
        int x,y,w,h;
        r.rect(&x, &y, &w, &h);
        bool sunken = on || down;
        int x2 = x+w-1;
        int y2 = y+h-1;
        QPen oldPen = p->pen();
        p->setPen(cdata->shades[5]);
	p->drawRect(x , y, w, h );

	p->setPen(cg.base());
        p->drawLine(x, y+1, x2-1, y+1);
        p->drawLine(x, y+1, x, y2-1);
	if (flags & Style_MouseOver)
	  renderGradient( p, QRect(x+2, y+2, w-3, h-3),
			  cdata->shades[2],cg.background(), !horizontal );
        else
	  renderGradient(p, QRect(x+2, y+2, w-3, h-3),
			 cg.background(),cdata->shades[2], !horizontal );
        p->setPen( oldPen );
        break;
      }
    case PE_IndicatorMask:
      {

        p->fillRect(r, color1);
        break;
      }
    case PE_ExclusiveIndicatorMask:
      {

        p->setPen(color1);
        p->setBrush(color1);
        p->drawPixmap(r.topLeft(), *cdata->radioMask);
        break;
      }
    case PE_ExclusiveIndicator:
      { //radiobutton
	int pix = 0;
	if (flags & Style_MouseOver) {
	  pix += 4;
	  p->fillRect(r, cg.brush(QColorGroup::Midlight));
        } else {
	  p->fillRect(r, cg.brush(QColorGroup::Background));
        }

        if (flags & Style_Down)
	  pix += 2;
        if (flags & Style_On)
	  pix += 1;

        p->drawPixmap(r.topLeft(), *cdata->radioPix[pix]);
	break;
      }
    case PE_CheckListIndicator:
      {
	QCheckListItem *item = opt.checkListItem();
	QListView *lv = item->listView();
	if(!item)
	  return;
	int x = r.x(), y = r.y(), w = r.width(), h = r.width(), marg = lv->itemMargin();
	drawPrimitive(PE_Indicator, p, QRect(x-2, y-2 , w+3, h+3 ) , cg, flags );
	break;
      }
    case PE_Indicator:
      { //radiobutton
        bool enabled  = flags & Style_Enabled;
        bool nochange = flags & Style_NoChange;
        drawFrame( cdata, cg, p, r, true );
        int x,y,w,h;
        x=r.x(); y=r.y(); w=r.width(); h=r.height();

	if (nochange) {
	  p->setPen(cdata->radioColor);
	  p->fillRect(x+2, y+2, w-4, h-4, cg.base());
	  p->drawLine(x+3, h/2, w-4, h/2);
	  p->drawLine(x+3, (h/2)+1, w-4, (h/2)+1);
	}
	else
	  {
	    if (xBmp.isNull()) {
	      xBmp = QBitmap(7, 7, x_bits, true);
	      xBmp.setMask(xBmp);
	    }
	    if ( enabled )
	      p->fillRect(x+2, y+2, w-4, h-4,
			  down ? cg.button(): cg.base());
	    else
	      p->fillRect(x+2, y+2, w-4, h-4, cdata->shades[1]);

	    if (!(flags & Style_Off)) {
	      if (on) {
		p->setPen(nochange ? /*cg.dark()*/ cdata->shades[2] : cdata->radioColor);
		p->drawPixmap(x+3, y+3, xBmp);
	      }
	      else {
		p->setPen(cg.shadow());
		p->drawRect(x+2, y+2, w-4, h-4);
		//FIXME !!!!!!!!!!!! cg.dark not good
		p->setPen(nochange ? cdata->radioColor : /*cg.dark()*/cdata->shades[2]);
		p->drawLine(x+3, (y+h)/2-2, x+w-4, (y+h)/2-2);
		p->drawLine(x+3, (y+h)/2, x+w-4, (y+h)/2);
		p->drawLine(x+3, (y+h)/2+2, x+w-4, (y+h)/2+2);
	      }
	    }
	  }
        break;
      }
    case PE_SpinWidgetUp:
    case PE_SpinWidgetDown:
      {
	p->save();
	QRect br;
	QRect re(r);
	if( pe == PE_SpinWidgetDown )
	  re.addCoords( 0, -1, 0, 0 );
	else
	  re.addCoords( 0, -1, 0, 0 );
        br.setRect( r.x(), r.y(), r.width(),
		    r.height());
	//#if 0 // FIXME gradient

        bool down = flags & Style_Down;
        bool on   = flags & Style_On;
        bool sunken = on || down;
        //cout <<" on :"<<on <<" down "<<down<<endl;
        //cout<<" sunken : "<<sunken<<endl;
        if (sunken)
	  renderGradient(p, /*QRect(x+1, y+1, w-2, h-2)*/br,
			 cdata->shades[0],cdata->shades[7], /*!horizontal*/ false);
        else if( !sunken && (flags & Style_Enabled ))
	  renderGradient(p, /*QRect(x+1  y+1, w-2, h-2)*/br,
			 cdata->shades[0],cdata->shades[7], /*!horizontal*/false );
#if 0
        else if( flags & Style_MouseOver )
	  renderGradient(p, /*QRect(x+1, y+1, w-2, h-2)*/br,
			 cg.base(), cdata->shades[1],/*!horizontal*/false );
#if 0
        renderGradient(p,r/*br*/,
		       cdata->shades[0],cdata->shades[2], false);
#endif
#endif
        p->setPen( cdata->shades[5]);
        p->drawRect(/*QRect( r.x()-1 + fw, r.y() + fw-1, r.width()+1 - fw*2,
                      r.height()+1 - fw*2 )*/re );
	//int x = r.x(), y = r.y(), h = r.height();
        int w = r.width();
        int sw = w-4;
	if ( sw < 3 )
	  break;
	else if ( !(sw & 1) )
	  sw--;
	sw -= ( sw / 7 ) * 2;	// Empty border
	//int sh = sw/2 + 2;      // Must have empty row at foot of arrow

	//int sx = x + w / 2 - sw / 2 - 1;
	//int sy = y + h / 2 - sh / 2 - 1;
	p->setPen( cg.buttonText() );
	p->setBrush( cg.buttonText() );
        QPointArray a;
        if ( pe == PE_SpinWidgetDown )
	  a.setPoints(QCOORDARRLEN(d_spinarrow), d_spinarrow);
        else
	  {
	    //move one pixel up
	    re.addCoords( 0, 1,0,0);
	    a.setPoints(QCOORDARRLEN(u_spinarrow), u_spinarrow);
	  }
        drawMandrakeArrow( a, p,
                           re,
                           cg,
                           flags,
                           opt );
	p->restore();

	break;
      }
    case PE_ScrollBarSlider: {
      // Small hack to ensure scrollbar gradients are drawn the right way.
      //flags ^= Style_Horizontal;
      //drawPrimitive(PE_ButtonBevel, p, r, cg, flags | Style_Enabled | Style_Raised);
      //draw slider border
      //oops not the place

      p->setPen( cdata->shades[5]);
      bool horizontal = flags & Style_Horizontal;

      if( scrollmin && scrollmax )
	{
          QRect tmpRect(r);
          if( !horizontal )
	    tmpRect.addCoords(0,0,0,1);
          else
	    tmpRect.addCoords(0,0,1,0);
          p->drawRect( tmpRect );
	}
      else
	p->drawRect( r );
      if (flags & Style_MouseOver)
        {
	  if( horizontal )
	    renderGradient(p, QRect( r.x()+1,scrollmin ? r.y() : r.y()+1,r.width()-2, scrollmax ? r.height()-1 : r.height()-2),
			   cg.base(), cdata->shades[1], false);
	  else
	    renderGradient(p, QRect( r.x()+1,scrollmin ? r.y() : r.y()+1,r.width()-2, scrollmax ? r.height()-1 : r.height()-2),
			   cg.base(), cdata->shades[1], true);

        }
      else
	{
	  //FIXME !!!!
	  if( !horizontal )
	    renderGradient(p, QRect( r.x()+1, scrollmin ? r.y() : r.y()+1,r.width()-2, scrollmax ? r.height()-1 : r.height()-2),
			   cdata->shades[0],cdata->shades[5], !horizontal);
	  else
	    renderGradient(p, QRect( scrollmin  ? r.x() : r.x()+1, r.y()+1, scrollmax ? r.width()-1 : r.width()-2, r.height()-2),
			   cdata->shades[0],cdata->shades[5], !horizontal);

	}
      int x,y,w,h;
      r.rect(&x, &y, &w, &h);
      int x2 = x+w-1;
      int y2 = y+h-1;

      p->setPen( cdata->shades[3]);
      if( !horizontal)
	p->drawLine( x2 - 1 , scrollmin ? y : y+1, x2 - 1, scrollmax ? y2 : y2-1 );
      p->setPen( cg.base());
      if( !horizontal )
	{
	  p->drawLine( x+1 , y, x+1 , scrollmax ? y2 : y2-1  );
          if( !scrollmin )
	    p->drawLine( x+1 , y, x2-1 , y  );
	}
      else //vertical error into horizontal...
	{
          if( !scrollmin )
            p->drawLine( x+1 , y+1, x+1 , scrollmax ? y2 : y2-1  );
          p->drawLine( x+1 , y+1, x2 -1 , y+1  );
	}
      break;
    }
    case PE_ScrollBarAddPage:
    case PE_ScrollBarSubPage: {
      int x, y, w, h;
      r.rect(&x, &y, &w, &h);
      int x2 = x+w-1;
      int y2 = y+h-1;

      p->setPen( cg.base());
      p->drawLine( x,y,x,y2 );
      p->drawLine( x2,y,x2,y2 );
      p->setPen( cdata->shades[5]);

      if (flags & Style_Horizontal) {
	p->drawLine(x, y, x2, y);
	p->drawLine(x, y2, x2, y2);
	renderGradient(p, QRect(x, y+1, w, h-2),
		       cdata->shades[2],cg.base(), false);
      } else {
	p->drawLine(x, y, x, y2);
	p->drawLine(x2, y, x2, y2);
	renderGradient(p, QRect(x+1, y, w-2, h),
		       cdata->shades[2],cg.base(), true);
      }
      break;
    }


    case PE_ScrollBarAddLine: {
      p->setPen( cdata->shades[5]);
      p->drawRect( r );
      bool horizontal = flags & Style_Horizontal;
      int x,y,w,h;
      r.rect(&x, &y, &w, &h);
      bool sunken = on || down;

      if (w > 4 && h > 4) {
	if (sunken)
	  renderGradient(p, QRect(x+1, y+1, w-2, h-2),
			 cdata->shades[1],cdata->shades[0], !horizontal);
	else if( !sunken && (flags & Style_Enabled ))
	  renderGradient(p, QRect(x+1, y+1, w-2, h-2),
			 cdata->shades[0],cdata->shades[5], !horizontal );
	else if( flags & Style_MouseOver )
	  renderGradient(p, QRect(x+1, y+1, w-2, h-2),
			 cg.base(), cdata->shades[1],!horizontal );
      }
      drawPrimitive( ((flags & Style_Horizontal) ? PE_ArrowRight : PE_ArrowDown),p, r, cg, flags );
      break;
    }
    case PE_ScrollBarSubLine: {
      p->setPen( cdata->shades[5]);
      p->drawRect( r );
      int x,y,w,h;
      r.rect(&x, &y, &w, &h);
      bool sunken = on || down;
      if (w > 4 && h > 4)
        {
	  bool horizontal = flags & Style_Horizontal;
	  if (sunken)
	    renderGradient(p, QRect(x+1, y+1, w-2, h-2),
			   cdata->shades[1],cdata->shades[0], !horizontal);
	  else if( !sunken && (flags & Style_Enabled ))
	    renderGradient(p, QRect(x+1, y+1, w-2, h-2),
			   cdata->shades[0],cdata->shades[5], !horizontal );
        }
      drawPrimitive( ((flags & Style_Horizontal) ? PE_ArrowLeft : PE_ArrowUp),
		     p, r, cg, flags );
      break;
    }


      // BUTTONS
      // -------------------------------------------------------------------
    case PE_ButtonDefault: //default button
    case PE_ButtonCommand: // action button
    case PE_ButtonDropDown:
    case PE_ButtonTool:
      {
        drawButton(cdata,r,p,cg,flags);
      }
      break;
    case PE_CheckMark:
      {
        if (menuBmp.isNull()) {
	  menuBmp = QBitmap( 12 , 12, menu_check_bits, true);
	  menuBmp.setMask(menuBmp);
        }
	int x,y;
	x=r.x(); y=r.y();

	if (!(flags & Style_Off)) {
	  p->setPen( on ? cdata->radioColor :  cg.highlightedText () );
	  p->drawPixmap(x+3, y+3, menuBmp);
	}

	break;
      }

      //FIXME !!!!!!!!!!!!!!!!!!!! scrollbar
    case PE_ButtonBevel: {
      int x,y,w,h;
      r.rect(&x, &y, &w, &h);
      bool sunken = on || down;
      int x2 = x+w-1;
      int y2 = y+h-1;

      // Outer frame
      p->setPen(cg.shadow());
      p->drawRect(r);

      // Bevel
      p->setPen(sunken ? cg.mid() : cg.light());
      p->drawLine(x+1, y+1, x2-1, y+1);
      p->drawLine(x+1, y+1, x+1, y2-1);
      p->setPen(sunken ? cg.light() : cg.mid());
      p->drawLine(x+2, y2-1, x2-1, y2-1);
      p->drawLine(x2-1, y+2, x2-1, y2-1);

      if (w > 4 && h > 4) {
	if (sunken)
	  p->fillRect(x+2, y+2, w-4, h-4, cg.button());
	/*else
	  renderGradient( p, QRect(x+2, y+2, w-4, h-4),
	  cg.button(), flags & Style_Horizontal );*/
      }
      break;
    }

    default:
      {
        if (pe >= PE_ArrowUp && pe <= PE_ArrowLeft)
	  {
            QPointArray a;
            switch(pe) {
            case PE_ArrowUp:
	      a.setPoints(QCOORDARRLEN(u_arrow), u_arrow);
	      break;

            case PE_ArrowDown:
	      a.setPoints(QCOORDARRLEN(d_arrow), d_arrow);
	      break;

            case PE_ArrowLeft:
	      a.setPoints(QCOORDARRLEN(l_arrow), l_arrow);
	      break;

            default:
	      a.setPoints(QCOORDARRLEN(r_arrow), r_arrow);
            }
            drawMandrakeArrow( a, p, r, cg, flags, opt );
	  }
        else
	  KStyle::drawPrimitive( pe, p, r, cg, flags, opt );
      }
    }

}


void MandrakeStyle::drawMandrakeArrow( QPointArray a, QPainter *p,
                                       const QRect &r,
                                       const QColorGroup &cg,
                                       SFlags flags,
                                       const QStyleOption& opt ) const
{
    QPen oldPen = p->pen();
    /*
      if ( flags & Style_Down )
      p->translate( pixelMetric( PM_ButtonShiftHorizontal ),
      pixelMetric( PM_ButtonShiftVertical ) );
    */
    if ( flags & Style_Enabled ) {
        a.translate( r.x() + r.width() / 2, r.y() + r.height() / 2 );
        p->setPen( cg.buttonText() );
        p->drawLineSegments( a );
    }
    else
    {
        a.translate( r.x() + r.width() / 2 + 1, r.y() + r.height() / 2 + 1 );
        p->setPen( cg.light() );
        p->drawLineSegments( a );
        a.translate( -1, -1 );
        p->setPen( cg.mid() );
        p->drawLineSegments( a );
    }
    p->setPen( oldPen );			// restore pen
}

void MandrakeStyle::drawKStylePrimitive( KStylePrimitive pe,
                                         QPainter* p,
                                         const QWidget* widget,
                                         const QRect &r,
                                         const QColorGroup &cg,
                                         SFlags flags,
                                         const QStyleOption &opt ) const
{
  const MandrakeColorData *cdata = lookupData(cg);
  switch(pe)
    {
      // GENERAL/KICKER HANDLE
      // -------------------------------------------------------------------
    case KPE_GeneralHandle:
      {
        drawHandle( cdata, p,r,cg,flags,opt,false );
        break;
      }

      // TOOLBAR HANDLE
      // -------------------------------------------------------------------
    case KPE_ToolBarHandle:
      {
        QRect real(r);
	real.addCoords( -1, 0,0,0);
        drawHandle( cdata, p,r,cg,flags,opt,true );
        break;
      }
      // SLIDER GROOVE
      // -------------------------------------------------------------------
    case KPE_SliderGroove: {
      const QSlider* slider = (const QSlider*)widget;
      bool horizontal = slider->orientation() == Horizontal;
      int gcenter = (horizontal ? r.height() : r.width()) / 2;

      QRect gr;
      if (horizontal)
	gr = QRect(r.x(), r.y()+gcenter-3, r.width(), 7);
      else
	gr = QRect(r.x()+gcenter-3, r.y(), 7, r.height());

      int x,y,w,h;
      gr.rect(&x, &y, &w, &h);
#if 0 //gradient doesn't work on small area :(((
      //FIXME gradient !!!!!
      renderGradient(p, QRect(x+2,y+2,w-4, h-4),
		     /*cg.base()*/cdata->shades[/*0*/2],cdata->shades[/*2*/0], !horizontal);
#endif
      bool enable  = flags & Style_Enabled;
      if( enable )
	{
	  if( horizontal )
	    {
	      p->setPen( cdata->shades[2]);
	      p->drawLine( x+2,y+2,w-2, y+2);
	      p->setPen( cdata->shades[1]);
	      p->drawLine( x+2,y+3,w-2, y+3);
	      p->setPen( cdata->shades[0]);
	      p->drawLine( x+2,y+4,w-2, y+4);
	    }
	  else
	    {
	      p->setPen( cdata->shades[2]);
	      p->drawLine( x+2,y+2,x+2, h-2);
	      p->setPen( cdata->shades[1]);
	      p->drawLine( x+3,y+2,x+3, h-2);
	      p->setPen( cdata->shades[0]);
	      p->drawLine( x+4,y+2,x+4, h-2);
	    }
	}
      p->setPen(enable ? cdata->shades[5] : cdata->shades[3] );
      p->drawRect(x+1, y+1, w-2, h-2);
      break;
    }

      // SLIDER HANDLE
      // -------------------------------------------------------------------
    case KPE_SliderHandle: {
      const QSlider* slider = (const QSlider*)widget;
      bool horizontal = slider->orientation() == Horizontal;
      int x,y,w,h;
      r.rect(&x, &y, &w, &h);
      int x2 = x+w-1;
      int y2 = y+h-1;
      p->save();
      bool enable  = flags & Style_Enabled;

      p->setPen( enable ? cdata->shades[5] : cdata->shades[3] );
      p->drawLine( x+1,y, x2-1,y);
      p->drawLine( x,y+1, x,y2-1);
      p->drawLine( x2,y+1, x2,y2-1);
      p->drawLine( x+1,y2, x2-1,y2);
      if( enable )
	renderGradient(p, QRect(x+1, y+1, w-2, h-2),
		       cdata->shades[0],cdata->shades[2], !horizontal);
      else
	p->fillRect( QRect(x+1, y+1, w-2, h-2 ), cg.background());

      p->setPen(cg.base());
      p->drawLine( x+1,y+1,x2-1,y+1);
      p->drawLine( x+1,y+1,x+1,y2-1);

      if( horizontal )
        {
	  p->drawLine( x+(x2-x)/2 +1 ,y+3,x+(x2-x)/2+1,y2-2);
	  p->setPen( cdata->shades[2]);
	  p->drawLine( x+(x2-x)/2 ,y+3,x+(x2-x)/2,y2-2);
        }
      else
        {
	  p->drawLine( x+1 ,y+(y2-y)/2+1, x2-2,y+(y2-y)/2+1);
	  p->setPen(  cdata->shades[2] );
	  p->drawLine( x+1 ,y+(y2-y)/2, x2-2,y+(y2-y)/2);

        }
      p->restore();
      break;
    }
    default:
      KStyle::drawKStylePrimitive( pe, p, widget, r, cg, flags, opt);
    }
}

void MandrakeStyle::drawControl( ControlElement element,
                                 QPainter *p,
                                 const QWidget *widget,
                                 const QRect &r,
                                 const QColorGroup &cg,
                                 SFlags flags,
                                 const QStyleOption& opt ) const
{
    const MandrakeColorData *cdata = lookupData(cg);
    switch (element)
    {
    case CE_ProgressBarContents: {
        const QProgressBar* pb = (const QProgressBar*)widget;
        QRect backend = subRect(SR_ProgressBarContents, widget);
        double progress = pb->progress();
        bool reverse = QApplication::reverseLayout();
        int steps = pb->totalSteps();

        if (!backend.isValid())
            return;
	p->fillRect( QRect( backend.x()+1, backend.y()+1, backend.width()-2, backend.height()-2), cdata->shades[2]);

        QRect cr(backend);
        cr.addCoords( 1, 1, -2, -2 );
       // Draw progress bar
        if (progress > 0 || steps == 0) {
            double pg = (steps == 0) ? 0.1 : progress / steps;
            int width = QMIN(cr.width(), (int)(pg * cr.width()));
            if (steps == 0)
                width = QMIN(width,20); //Don't cross squares

            if (steps == 0) { //Busy indicator

                if (width < 1) width = 1; //A busy indicator with width 0 is kind of useless

                int remWidth = cr.width() - width; //Never disappear completely
                if (remWidth <= 0) remWidth = 1; //Do something non-crashy when too small...

                int pstep =  int(progress) % ( 2 *  remWidth );

                if ( pstep > remWidth ) {
                    //Bounce about.. We're remWidth + some delta, we want to be remWidth - delta...
                    // - ( (remWidth + some delta) - 2* remWidth )  = - (some deleta - remWidth) = remWidth - some delta..
                    pstep = - (pstep - 2 * remWidth );
                }
                if( reverse )
                    renderGradient(p, QRect(cr.x() + cr.width() - width - pstep, cr.y(), width, cr.height()),
                                   cdata->shades[2],cdata->shades[1], false);
                else
                    renderGradient(p, cr,
                                   cdata->shades[2],cdata->shades[1], false);
		p->setPen(cdata->shades[5]);
		p->drawLine( backend.x(), backend.y(), backend.width()+1, backend.y());
		p->drawLine( backend.x(), backend.y(), backend.x(), backend.height());
                return;

            }
            if( reverse )
            {
                renderGradient(p, QRect( cr.x()+(cr.width()-width), cr.y(), width, cr.height()),
                               cdata->shades[1], cdata->blueMdkColor[0], false);

                int x = cr.x()+(cr.width()-width);
                int y = cr.y();
                int x2 = width+x;
                int y2 = cr.y() + cr.height();

                p->setPen( cg.base());
                p->drawLine( x,y,x2,y);
                p->drawLine( x2,y,x2,y2);

                p->setPen( cdata->blueMdkColor[0]);
                p->drawLine( x,y+1,x,y2);
                p->drawLine( x,y2,x2-1,y2);
            }
            else
            {
                renderGradient(p, QRect(cr.x(), cr.y(), width, cr.height()),
                               cdata->shades[1],cdata->blueMdkColor[0], false);
                int x = cr.x();
                int y = cr.y();
                int x2 = cr.x() + width;
                int y2 = cr.y() + cr.height();
                p->setPen( cg.base());
                p->drawLine( x,y,x2,y);
                p->drawLine( x,y+1,x,y2);

                p->setPen( cdata->blueMdkColor[0]);
                p->drawLine( x2,y+1,x2,y2-1);
                p->drawLine( x+1,y2-1,x2,y2-1);
            }
        }
        p->setPen(cdata->shades[5]);
	p->drawLine( backend.x(), backend.y(), backend.width(), backend.y());
	p->drawLine( backend.x(), backend.y(), backend.x(), backend.height());

        break;
    }
    case CE_ToolButtonLabel:
    {
        KStyle::drawControl(element, p, widget, r, cg, flags, opt);
#if 0
        //const QToolButton *toolbutton = static_cast<const QToolButton * >(widget);
        bool onToolbar = widget->parentWidget() && widget->parentWidget()->inherits( "QToolBar" );
        QRect nr = r;
	int  x, y, w, h;
	r.rect(&x, &y, &w, &h);

	int x2 = x+w-1;
	int y2 = y+h-1;

        if (!onToolbar)
        {
            if (widget->parentWidget() &&
                !qstrcmp(widget->parentWidget()->name(),"qt_maxcontrols" ) )
            {
                //Make sure we don't cut into the endline
                if (!qstrcmp(widget->name(), "close"))
                {
                    nr.addCoords(0,0,-1,0);
                    p->setPen(cg.dark());
                    p->drawLine(r.x() + r.width() - 1, r.y(),
                                r.x() + r.width() - 1, r.y() + r.height() - 1 );
                }
            }
            else if ( w > smallButMaxW && h > smallButMaxH )
                nr.setWidth(r.width()-2); //Account for shadow
        }

        KStyle::drawControl(element, p, widget, nr, cg, flags, opt);
#endif
        break;
    }

    //TODO FIXME
#if (QT_VERSION-0 >= 0x030100)
    case CE_MenuBarEmptyArea:
    {
        p->fillRect(r, cg.brush(QColorGroup::Button));
        break;
    }
    case CE_DockWindowEmptyArea:
    {
      p->fillRect(r, cg.brush(QColorGroup::Button));
        break;
    }
#endif
    case CE_TabBarTab:
    {
        const QTabBar* tb = static_cast< const QTabBar* >( widget );
        bool selected = flags & Style_Selected;
        //bool bottom = tb->shape() == QTabBar::RoundedBelow || tb->shape() == QTabBar::TriangularBelow;

        int x, y, xw, yh;
        int offset= 0;
        x= r.x();
        y= r.y();
        xw = r.x()+r.width()-1;
        yh = r.y()+r.height()-1;

        if ( selected )
        {
            p->fillRect( r, tb->palette().active().brush( QColorGroup::Background ));
            p->setPen( cdata->shades[5]);
            p->drawLine( xw,y+2,xw,yh);
        }
        else
        {
            offset = 2;
            p->fillRect( QRect(r.x()+2,r.y()+offset+2,r.width()-2,r.height()-offset-2), cdata->shades[2]);
            p->setPen( cdata->shades[5]);
            p->drawLine( xw,y+offset+2,xw,yh+offset-2);
            //if( bottom ) no necessary
            {
                p->setPen( cg.base() );
                p->drawLine( x,yh/*+offset-2*/,xw,yh/*+offset-2*/);
            }
        }
        p->setPen( cdata->shades[5]);
        p->drawLine( x+2,y+offset,xw-2,y+offset);
        p->drawPoint( x+1,y+offset+1);
        p->drawPoint( xw-1,y+offset+1);

        p->setPen(selected ? cg.base() : cdata->shades[5]);
        p->drawLine( x+2,y+offset,xw-2,y+offset);
        p->drawLine( x,y+offset+2,x,yh/*-offset*/);
        p->drawPoint( x+1,y+offset+1);
        break;
    }
    case CE_MenuBarItem:
    {
        QMenuBar  *mb = (QMenuBar*)widget;
        QMenuItem *mi = opt.menuItem();
        QRect      pr = mb->rect();

        bool active  = flags & Style_Active;
        bool focused = flags & Style_HasFocus;

        if ( active && focused )
            qDrawShadePanel(p, r.x(), r.y(), r.width(), r.height(),
                            cg, true, 1, &cg.brush(QColorGroup::Midlight));
        else
            renderGradient(p, r, cg.button(), cg.button(),false,
                            r.x(), r.y()-1, pr.width()-2, pr.height()-2);

        drawItem( p, r, AlignCenter | AlignVCenter | ShowPrefix
                  | DontClip | SingleLine, cg, flags & Style_Enabled,
                  mi->pixmap(), mi->text() );

        break;
    }


    // POPUPMENU ITEM
    // -------------------------------------------------------------------
    case CE_PopupMenuItem:
    {
        const QPopupMenu *popupmenu = (const QPopupMenu *) widget;

        QMenuItem *mi = opt.menuItem();
        if ( !mi )
        {
            // Don't leave blank holes if we set NoBackground for the QPopupMenu.
            // This only happens when the popupMenu spans more than one column.
            if (! (widget->erasePixmap() && !widget->erasePixmap()->isNull()) )
                p->fillRect(r, cg.brush(QColorGroup::Button) );
            break;
        }

        int  tab        = opt.tabWidth();
        int  checkcol   = opt.maxIconWidth();
        bool enabled    = mi->isEnabled();
        bool checkable  = popupmenu->isCheckable();
        bool active     = flags & Style_Active;
        bool etchtext   = styleHint( SH_EtchDisabledText );
        bool reverse    = QApplication::reverseLayout();
        int x, y, w, h;
        r.rect( &x, &y, &w, &h );

        if ( checkable )
            checkcol = QMAX( checkcol, 20 );

        // Are we a menu item separator?
        if ( mi->isSeparator() )
        {
	        p->setPen(  cdata->shades[2] );
            p->drawLine( x, y, x+w, y );
            p->setPen( cg.base() );
            p->drawLine( x, y+1, x+w, y+1 );
            break;
        }
        // Draw the menu item background
	if( active && (flags & Style_Enabled))
	  renderGradient(p, r, cdata->blueMdkColor[0], cdata->blueMdkColor[2],false);
        // Draw the transparency pixmap
        else if ( widget->erasePixmap() && !widget->erasePixmap()->isNull() )
            p->drawPixmap( x, y, *widget->erasePixmap(), x, y, w, h );
        // Draw a solid background
        else
            p->fillRect( r, cg.button() );


        // Do we have an icon?
#if 0
	if ( (  (widget && !qstrcmp(widget->name(),"KMenu" )) ||
	      (widget->parentWidget() && !qstrcmp(widget->parentWidget()->name(),"KMenu" )) )
	      &&
#endif
	     if( mi->iconSet() )
        {
            QIconSet::Mode mode;
            QRect cr = visualRect( QRect(x, y, checkcol, h), r );

            // Select the correct icon from the iconset
            if ( active )
                mode = enabled ? QIconSet::Active : QIconSet::Disabled;
            else
                mode = enabled ? QIconSet::Normal : QIconSet::Disabled;
#if 0 //don't check menu
            // Do we have an icon and are checked at the same time?
            // Then draw a "pressed" background behind the icon
            if ( checkable && !active && mi->isChecked() )
	      {
                qDrawShadePanel( p, cr.x(), cr.y(), cr.width(), cr.height(),
                                 cg, true, 1, &cg.brush(QColorGroup::Midlight) );
	      }
#endif
	    //TEST IT !!!!!!!!!!!!!!
	    if ( checkable && mi->isChecked() )
	      {//don't display icon when button is checkable
		int cx = reverse ? x+w - checkcol : x;
		// Draw the checkmark
		SFlags cflags = Style_Default;
		cflags |= active ? Style_Enabled : Style_On;
		drawPrimitive( PE_CheckMark, p, QRect( cx + itemFrame, y + itemFrame,
						       checkcol - itemFrame*2, h - itemFrame*2), cg, cflags );
	      }
	    else
	      {
		// Draw the icon
		QPixmap pixmap = mi->iconSet()->pixmap( QIconSet::Small, mode );
		QRect pmr( 0, 0, pixmap.width(), pixmap.height() );
		pmr.moveCenter( cr.center() );
		p->drawPixmap( pmr.topLeft(), pixmap );
	      }
        }
        // Are we checked? (This time without an icon)
        else if (  (checkable && mi->isChecked()) )
        {
            int cx = reverse ? x+w - checkcol : x;
#if 0 //don't create a checked button
            // We only have to draw the background if the menu item is inactive
            // if it's active the "pressed" background is already drawn
            if ( ! active )
                qDrawShadePanel( p, cx, y, checkcol, h, cg, true, 1,
                                 &cg.brush(QColorGroup::Midlight) );
#endif
            // Draw the checkmark
            SFlags cflags = Style_Default;
            cflags |= active ? Style_Enabled : Style_On;
            drawPrimitive( PE_CheckMark, p, QRect( cx + itemFrame, y + itemFrame,
                                                   checkcol - itemFrame*2, h - itemFrame*2), cg, cflags );
        }

        // Time to draw the menu item label...
        int xm = itemFrame + checkcol + itemHMargin; // X position margin

        int xp = reverse ? // X position
                 x + tab + rightBorder + itemHMargin + itemFrame - 1 :
                 x + xm;

        int offset = reverse ? -1 : 1;	// Shadow offset for etched text

        // Label width (minus the width of the accelerator portion)
        int tw = w - xm - tab - arrowHMargin - itemHMargin * 3 - itemFrame + 1;

        // Set the color for enabled and disabled text
        // (used for both active and inactive menu items)
        p->setPen( enabled ? cg.buttonText() : cg.mid() );

        // This color will be used instead of the above if the menu item
        // is active and disabled at the same time. (etched text)
        QColor discol = cg.mid();

        // Does the menu item draw it's own label?
        if ( mi->custom() )
        {
            int m = itemVMargin;
            // Save the painter state in case the custom
            // paint method changes it in some way
            QPen oldPen = p->pen();


            // Draw etched text if we're inactive and the menu item is disabled
            if ( etchtext && !enabled && !active )
            {
                p->setPen( cg.light() );
                mi->custom()->paint( p, cg, active, enabled, xp+offset, y+m+1, tw, h-2*m );
                p->setPen( discol );
            }
            mi->custom()->paint( p, cg, active, enabled, xp, y+m, tw, h-2*m );
            p->setPen( oldPen );			// restore pen

        }
        else
        {
            // The menu item doesn't draw it's own label
            QString s = mi->text();

            // Does the menu item have a text label?
            if ( !s.isNull() ) {
                int t = s.find( '\t' );
                int m = itemVMargin;
                int text_flags = AlignVCenter | ShowPrefix | DontClip | SingleLine;
                text_flags |= reverse ? AlignRight : AlignLeft;

                //QColor draw = cg.text();
                QColor draw = (active && enabled) ? cg.highlightedText () : cg.text();
                p->setPen(draw);


                // Does the menu item have a tabstop? (for the accelerator text)
                if ( t >= 0 )
                {
                    int tabx = reverse ? x + rightBorder + itemHMargin + itemFrame :
                               x + w - tab - rightBorder - itemHMargin - itemFrame;

                    // Draw the right part of the label (accelerator text)
                    if ( etchtext && !enabled && !active )
                    {
                        // Draw etched text if we're inactive and the menu item is disabled
                        p->setPen( cg.light() );
                        p->drawText( tabx+offset, y+m+1, tab, h-2*m, text_flags, s.mid( t+1 ) );
                        p->setPen( discol );
                    }
                    p->drawText( tabx, y+m, tab, h-2*m, text_flags, s.mid( t+1 ) );
                    s = s.left( t );
                }

                // Draw the left part of the label (or the whole label
                // if there's no accelerator)
                if ( etchtext && !enabled && !active )
                {
                    // Etched text again for inactive disabled menu items...
                    p->setPen( cg.light() );
                    p->drawText( xp+offset, y+m+1, tw, h-2*m, text_flags, s, t );
                    p->setPen( discol );
                }

                p->drawText( xp, y+m, tw, h-2*m, text_flags, s, t );

            }

            // The menu item doesn't have a text label
            // Check if it has a pixmap instead
            else if ( mi->pixmap() )
            {
                QPixmap *pixmap = mi->pixmap();

                // Draw the pixmap
                if ( pixmap->depth() == 1 )
                    p->setBackgroundMode( OpaqueMode );

                int diffw = ( ( w - pixmap->width() ) / 2 )
                            + ( ( w - pixmap->width() ) % 2 );
                p->drawPixmap( x+diffw, y+itemFrame, *pixmap );

                if ( pixmap->depth() == 1 )
                    p->setBackgroundMode( TransparentMode );
            }
        }

        // Does the menu item have a submenu?
        if ( mi->popup() )
        {
            PrimitiveElement arrow = reverse ? PE_ArrowLeft : PE_ArrowRight;
            int dim = pixelMetric(PM_MenuButtonIndicator)- itemFrame;
            QRect vr = visualRect( QRect( x + w - arrowHMargin - itemFrame - dim,
                                          y + h / 2 - dim / 2, dim, dim), r );

            // Draw an arrow at the far end of the menu item
            if ( active )
            {
                if ( enabled )
                    discol = cg.buttonText();

                QColorGroup g2( cg.highlightedText ()/*white*/, cg.highlight(), /*white*/cg.highlightedText (), /*white*/cg.highlightedText (),
                                enabled ? cg.highlightedText () : discol, /*discol*/cg.highlightedText (), cg.highlightedText () );

                drawPrimitive( arrow, p, vr, g2, Style_Enabled );
            }
            else
                drawPrimitive( arrow, p, vr, cg,
                               enabled ? Style_Enabled : Style_Default );
        }
        break;
    }
    case CE_PushButtonLabel:
    {
        const QPushButton* button = (const QPushButton*)widget;
        bool active = button->isOn() || button->isDown();
        int x, y, w, h;
        r.rect( &x, &y, &w, &h );

        // Shift button contents if pushed.
        if ( active )
        {
            x += pixelMetric(PM_ButtonShiftHorizontal, widget);
            y += pixelMetric(PM_ButtonShiftVertical, widget);
            flags |= Style_Sunken;
        }

        // Does the button have a popup menu?
        if ( button->isMenuButton() )
        {
            int dx = pixelMetric( PM_MenuButtonIndicator, widget );
            drawPrimitive( PE_ArrowDown, p, QRect(x + w - dx - 2, y + 2, dx, h - 4),
                           cg, flags, opt );
            w -= dx;
        }

        // Draw the icon if there is one
        if ( button->iconSet() && !button->iconSet()->isNull() ) {
            QIconSet::Mode  mode  = QIconSet::Disabled;
            QIconSet::State state = QIconSet::Off;

            if (button->isEnabled())
                mode = button->hasFocus() ? QIconSet::Active : QIconSet::Normal;
            if (button->isToggleButton() && button->isOn())
                state = QIconSet::On;

            QPixmap pixmap = button->iconSet()->pixmap( QIconSet::Small, mode, state );

            // Center the iconset if there's no text or pixmap
            if (button->text().isEmpty() && !button->pixmap())
                p->drawPixmap( x + (w - pixmap.width())  / 2,
                               y + (h - pixmap.height()) / 2, pixmap );
            else
                p->drawPixmap( x + 4, y + (h - pixmap.height()) / 2, pixmap );

            int  pw = pixmap.width();
            x += pw + 4;
            w -= pw + 4;
        }
        // Make the label indicate if the button is a default button or not
        if ( active || button->isDefault() ) {
            // Draw "fake" bold text  - this enables the font metrics to remain
            // the same as computed in QPushButton::sizeHint(), but gives
            // a reasonable bold effect.
            drawItem( p, QRect(x, y, w, h), AlignCenter | ShowPrefix,
                      button->colorGroup(),
                      button->isEnabled(),
                      button->pixmap(),
                      button->text(), -1,
                      &button->colorGroup().buttonText() );

#if 0
            // Normal Text
            for(i=0; i<2; i++)
                drawItem( p, QRect(x+i, y, w, h), AlignCenter | ShowPrefix,
                          button->colorGroup(),
                          button->isEnabled(),
                          i == 0 ? button->pixmap() : NULL,
                          button->text(), -1,
                          &button->colorGroup().buttonText() );
#endif
        } else
        {
            drawItem( p, QRect(x, y, w, h), AlignCenter | ShowPrefix,
                      button->colorGroup(),
                      button->isEnabled(), button->pixmap(),
                      button->text(), -1,
                      &button->colorGroup().buttonText() );
        }
        // Draw a focus rect if the button has focus
        if ( flags & Style_HasFocus )
            drawPrimitive( PE_FocusRect, p,
                           QStyle::visualRect(subRect(SR_PushButtonFocusRect, widget), widget),
                           cg, flags );
        break;
    }
    case CE_PushButton:
    {
        const QPushButton* btn = static_cast< const QPushButton* >( widget );
        if ( widget == hoverWidget )
            flags |= Style_MouseOver;


        if ( btn->isDefault( ))
        {
            drawPrimitive( PE_ButtonDefault, p, r, cg, flags );
        }
        else
            drawPrimitive( PE_ButtonCommand, p, r, cg, flags );
    }
    break;
    default:
        KStyle::drawControl(element, p, widget, r, cg, flags, opt);
    }
}
void MandrakeStyle::drawControlMask( ControlElement element,
                                     QPainter *p,
                                         const QWidget *widget,
                                         const QRect &r,
                                         const QStyleOption& opt ) const
{
    switch (element)
    {
        // PUSHBUTTON MASK
        // ------------------------------------------------------------------
    case CE_PushButton: {
        int x1, y1, x2, y2;
        r.coords( &x1, &y1, &x2, &y2 );
        QCOORD corners[] = { x1,y1, x2,y1, x1,y2, x2,y2 };
        p->fillRect( r, color1 );
        p->setPen( color0 );
        p->drawPoints( QPointArray(4, corners) );
        break;
    }

    default:
        KStyle::drawControlMask(element, p, widget, r, opt);
    }
}

void MandrakeStyle::drawComboboxArrow( QPainter *p,
                                       const QWidget *widget,
                                       const QRect &ar,
                                       const QColorGroup &cg,
                                       SFlags flags,
                                       SCFlags active)const
{
    const QComboBox * cb = (const QComboBox *) widget;
    bool editableCombo = cb->editable();
    // Are we enabled?
    if ( widget->isEnabled() )
        flags |= Style_Enabled;

    // Are we "pushed" ?
    if ( active & Style_Sunken )
        flags |= Style_Sunken;

#if 0//Pb with focus
    if ( cb->hasFocus() ) {
        p->setPen( cg.highlightedText() );
        //p->setBackgroundColor( cg.highlight() );
    } else
#endif
    {

        p->setPen( cg.text() );
        //Don't apply background color otherwise it
        //destroys gradient.
        //p->setBackgroundColor( cg.button() );
    }
    QPointArray a;
    a.setPoints(QCOORDARRLEN(c_arrow), c_arrow);

    if ( flags & Style_Down )
        p->translate( pixelMetric( PM_ButtonShiftHorizontal ),
                      pixelMetric( PM_ButtonShiftVertical ) );

    if ( flags & Style_Enabled ) {
      a.translate( editableCombo ? ar.x() + ar.width() / 2  : ar.x() + ar.width() / 2-2, editableCombo ? ar.y() + ar.height() / 2 + 1 : ar.y() + ar.height() / 2 );
        p->setPen( cg.buttonText() );
        p->drawLineSegments( a );
    }
    else
    {
      a.translate( editableCombo ? ar.x() + ar.width() / 2 + 1 : ar.x() + ar.width() / 2 -2, editableCombo ? ar.y() + ar.height() / 2  + 1:  ar.y() + ar.height() / 2  );
        p->setPen( cg.light() );
        p->drawLineSegments( a );
        a.translate( -1, -1 );
        p->setPen( cg.mid() );
        p->drawLineSegments( a );
    }
}

void MandrakeStyle::drawComplexControl( ComplexControl control,
                                            QPainter *p,
                                            const QWidget *widget,
                                            const QRect &r,
                                            const QColorGroup &cg,
                                            SFlags flags,
                                            SCFlags controls,
                                            SCFlags active,
                                            const QStyleOption& opt ) const
{
    const MandrakeColorData *cdata = lookupData(cg);
    switch(control)
    {
    case CC_SpinWidget:
    {
        const QSpinWidget* sw = static_cast< const QSpinWidget* >( widget );
	QRect br = visualRect( querySubControlMetrics( CC_SpinWidget, widget, SC_SpinWidgetButtonField ), widget );
        br.addCoords( 1,0,1,2);
        if ( controls & SC_SpinWidgetFrame )
            drawPrimitive( PE_PanelLineEdit, p, r, cg );
        if ( controls & SC_SpinWidgetButtonField )
        {
            if ( controls  & SC_SpinWidgetUp)
	      {
		//fixme color not good :(((
		QColorGroup ucg = sw->isUpEnabled() ? cg : sw->palette().disabled();

                drawPrimitive( PE_SpinWidgetUp,p, QRect(br.x()-1 , br.y() , br.width() + 1, br.height() / 2), ucg, flags, opt);
	      }
            if ( controls & SC_SpinWidgetDown)
	      {
		QColorGroup dcg = sw->isDownEnabled() ? cg : sw->palette().disabled();

                drawPrimitive( PE_SpinWidgetDown,p, QRect( br.x()-1 , br.y() + br.height() / 2 , br.width() + 1, br.height()/ 2  ), dcg, flags, opt);
	      }
        }
        break;
    }
    case CC_ScrollBar:
    {
        bool bHover = (widget == hoverWidget);
        const QScrollBar *scrollbar = (const QScrollBar *) widget;
        QRect addline, subline, addpage, subpage, slider, first, last;
        bool maxedOut = (scrollbar->minValue() == scrollbar->maxValue());
	scrollmin = ( scrollbar->minValue() == scrollbar->value() );
        //cout <<"scrollbar->maxValue() :"<<scrollbar->maxValue()<<" scrollbar->minValue() :"<<scrollbar->minValue()<<" scrollbar->value() :"<<scrollbar->value()<<endl;
	scrollmax = (( scrollbar->maxValue() == scrollbar->value() )|| maxedOut);
        //cout<<" scrollmax :"<<scrollmax<<endl;
	subline = querySubControlMetrics(control, widget, SC_ScrollBarSubLine, opt);
        addline = querySubControlMetrics(control, widget, SC_ScrollBarAddLine, opt);
        subpage = querySubControlMetrics(control, widget, SC_ScrollBarSubPage, opt);
        addpage = querySubControlMetrics(control, widget, SC_ScrollBarAddPage, opt);
        slider  = querySubControlMetrics(control, widget, SC_ScrollBarSlider,  opt);

        first   = querySubControlMetrics(control, widget, SC_ScrollBarFirst,   opt);
        last    = querySubControlMetrics(control, widget, SC_ScrollBarLast,    opt);

        if ((controls & SC_ScrollBarSubLine) && subline.isValid()) {
            drawPrimitive(PE_ScrollBarSubLine, p, subline, cg,
                          Style_Enabled | ((active == SC_ScrollBarSubLine) ?
                                           Style_Down : Style_Default) |
                          ((scrollbar->orientation() == Qt::Horizontal) ?
                           Style_Horizontal : 0));

        }
        if ((controls & SC_ScrollBarAddLine) && addline.isValid())
            drawPrimitive(PE_ScrollBarAddLine, p, addline, cg,
                          Style_Enabled | ((active == SC_ScrollBarAddLine) ?
                                           Style_Down : Style_Default) |
                          ((scrollbar->orientation() == Qt::Horizontal) ?
                           Style_Horizontal : 0));
        if ((controls & SC_ScrollBarSubPage) && subpage.isValid())
            drawPrimitive(PE_ScrollBarSubPage, p, subpage, cg,
                          Style_Enabled | ((active == SC_ScrollBarSubPage) ?
                                           Style_Down : Style_Default) |
                          ((scrollbar->orientation() == Qt::Horizontal) ?
                           Style_Horizontal : 0));
        if ((controls & SC_ScrollBarAddPage) && addpage.isValid())
            drawPrimitive(PE_ScrollBarAddPage, p, addpage, cg,
                          ((maxedOut) ? Style_Default : Style_Enabled) |
                          ((active == SC_ScrollBarAddPage) ?
                           Style_Down : Style_Default) |
                          ((scrollbar->orientation() == Qt::Horizontal) ?
                           Style_Horizontal : 0));
        if ((controls & SC_ScrollBarFirst) && first.isValid())
            drawPrimitive(PE_ScrollBarFirst, p, first, cg,
                          Style_Enabled | ((active == SC_ScrollBarFirst) ?
                                           Style_Down : Style_Default) |
                          ((scrollbar->orientation() == Qt::Horizontal) ?
                           Style_Horizontal : 0));
        if ((controls & SC_ScrollBarLast) && last.isValid())
            drawPrimitive(PE_ScrollBarLast, p, last, cg,
                          Style_Enabled | ((active == SC_ScrollBarLast) ?
                                           Style_Down : Style_Default) |
                          ((scrollbar->orientation() == Qt::Horizontal) ?
                           Style_Horizontal : 0));
        if ((controls & SC_ScrollBarSlider) && slider.isValid()) {
            drawPrimitive(PE_ScrollBarSlider, p, slider, cg,
                          /*bHover ? Style_MouseOver : 0 |*/
                          Style_Enabled | ((active == SC_ScrollBarSlider) ?
                                           Style_Down : Style_Default) |
                          ((scrollbar->orientation() == Qt::Horizontal) ?
                           Style_Horizontal : 0));

            // ### perhaps this should not be able to accept focus if maxedOut?
            if (scrollbar->hasFocus()) {
                QRect fr(slider.x() + 2, slider.y() + 2,
                         slider.width() - 5, slider.height() - 5);
                drawPrimitive(PE_FocusRect, p, fr, cg, Style_Default);
            }
        }
        break;
    }
    // COMBOBOX
    // -------------------------------------------------------------------
    case CC_ComboBox: {
        const QComboBox * cb = (const QComboBox *) widget;
        // Draw box and arrow
        bool sunken = (active == SC_ComboBoxArrow);
        bool reverse = QApplication::reverseLayout();
	if ( widget == hoverWidget )
	  flags |= Style_MouseOver;

        if ( !cb->editable())
        {
            QPen oldPen = p->pen();

            // Get the button bounding box
            QRect ar = QStyle::visualRect(querySubControlMetrics(CC_ComboBox, widget, SC_ComboBoxArrow), widget );
	    //FIXME !!!!!!!!!!!!!!
	    //r.setCoords( 0, 1, -1, -1 );
            if( sunken )
                renderGradient(p, QRect(ar.x()+4, ar.y()+4, ar.width()-8, ar.height()-10), cdata->shades[2], cdata->shades[1],false);
            else if( !sunken && (widget->isEnabled()))
                renderGradient(p, QRect(ar.x()+4, ar.y()+4, ar.width()-8, ar.height()-10), cdata->shades[1], cdata->shades[2],false);

            drawButton( cdata,r,p,cg,flags);

            if( reverse )
            {
                p->setPen(cg.base());
                p->drawLine( ar.width()-1,ar.y()+4,ar.width()-1,ar.height()-6);
                p->setPen( cdata->shades[2]);
                p->drawLine( ar.width(),ar.y()+4,ar.width(),ar.height()-6);
            }
            else
            {
                p->setPen(cg.base());
                p->drawLine( ar.x()-1,ar.y()+4,ar.x()-1,ar.height()-6);
                p->setPen(cdata->shades[2]);
                p->drawLine( ar.x()-2,ar.y()+4,ar.x()-2,ar.height()-6);
            }

            drawComboboxArrow( p,widget,ar,cg,flags,active);
            p->setPen( oldPen );			// restore pen

        }
        // Draw an edit field if required
        else
        {
            //int x, y, xw, yh;
            QRect er = visualRect( querySubControlMetrics( CC_ComboBox, widget, SC_ComboBoxFrame ), widget );
            drawPrimitive( PE_PanelLineEdit, p, er, cg );

            er = visualRect( querySubControlMetrics( CC_ComboBox, widget, SC_ComboBoxEditField ), widget );
            if((flags & Style_Enabled ))
                er.addCoords( -4, -2, 6, 4 );
            p->fillRect( er, cg.base() );

            // Get the button bounding box
            QRect ar = QStyle::visualRect(
                querySubControlMetrics(CC_ComboBox, widget, SC_ComboBoxArrow),
                widget );
	    bool mouseOver = (flags & Style_MouseOver);
	    if ( mouseOver && !sunken )
	      renderGradient(p, QRect(ar.x()+1, ar.y()+1, ar.width()-1, ar.height()-2),
			     cg.base(), cdata->shades[1], false);

            else if( !mouseOver )
	      {
		if( sunken )
		  renderGradient(p, QRect(ar.x()+1, ar.y()+1, ar.width()-1, ar.height()-2),
				 cdata->shades[2],cdata->shades[1], false);
		else
		  renderGradient(p, QRect(ar.x()+1, ar.y()+1, ar.width()-1, ar.height()-2),
				 cdata->shades[1], cdata->shades[2], false);
	      }
	    ar.addCoords( 1,1,0,-1);
	    p->setPen( cdata->shades[5]);
	    p->drawRect( ar );
            drawComboboxArrow( p,widget,ar,cg,flags,active);


            break;
        }
    }
    default:
        KStyle::drawComplexControl(control, p, widget,
                                   r, cg, flags, controls, active, opt);
    }
}

//todo add "querySubControl" !!!!!!!!!!!!!!!!!!!!!!!!!
QRect MandrakeStyle::querySubControlMetrics( ComplexControl control,
                                            const QWidget* widget,
                                            SubControl subcontrol,
                                            const QStyleOption& opt ) const
{
    switch (control)
    {
    case CC_ComboBox:
    {
        int arrow = 11;
	int fw = 0;
	if( static_cast< const QComboBox* >( widget )->editable())
	  fw = 2;

        switch ( subcontrol )
        {
	case SC_ComboBoxFrame:
	  return QRect( 0, 0, widget->width(), widget->height()-fw  );
        case SC_ComboBoxArrow:
            return QRect( widget->width() - arrow - 14, 0, arrow+13, widget->height()-fw );
        case SC_ComboBoxEditField:
        {
            if ( widget->width() < 36 || widget->height() < 22 )
                return QRect( 4, 3, widget->width() - arrow-20, widget->height()-fw );
            else if ( static_cast< const QComboBox* >( widget )->editable() )
	      return QRect(  6,4, widget->width() - arrow-25, widget->height() - 10 -fw );
            else
	      return QRect( 6, 4, widget->width() - arrow-22, widget->height() - 8 -fw);
        }
        default:
            return KStyle::querySubControlMetrics( control, widget, subcontrol, opt );
        }
        break;
    }
    case CC_Slider:
    {
        const QSlider* sl = static_cast< const QSlider* >( widget );
        bool horizontal = sl->orientation() == Horizontal;
        QSlider::TickSetting ticks = sl->tickmarks();
        int pos = sl->sliderStart();
        //int size = pixelMetric( PM_SliderControlThickness, widget );
        int handleSize = pixelMetric( PM_SliderThickness, widget );
        int len = pixelMetric( PM_SliderLength, widget );
        switch ( subcontrol )
        {
        case SC_SliderHandle:
            if ( horizontal )
            {
                if ( ticks == QSlider::Both )
                    return QRect( pos, ( sl->height() - handleSize ) / 2, len, handleSize );
                else if ( ticks == QSlider::Above )
                    return QRect( pos, sl->height() - handleSize, len, handleSize );
                return QRect( pos, 0, len, handleSize );
            }
            else
            {
                if ( ticks == QSlider::Both )
                    return QRect( ( sl->width() - handleSize ) / 2, pos, handleSize, len );
                else if ( ticks == QSlider::Above )
                    return QRect( sl->width() - handleSize, pos, handleSize, len );
                return QRect( 0, pos, handleSize, len );
            }
        default:
            break;
        }
        break;
    }
    default:
        break;
    }
    return KStyle::querySubControlMetrics( control, widget, subcontrol, opt );
}

void MandrakeStyle::drawComplexControlMask( ComplexControl control,
                                            QPainter *p,
                                            const QWidget *widget,
                                            const QRect &r,
                                            const QStyleOption& opt ) const
{
    switch (control)
    {
        // COMBOBOX & TOOLBUTTON MASKS
        // -------------------------------------------------------------------
    case CC_ComboBox:
    case CC_ToolButton: {
        int x1, y1, x2, y2;
        r.coords( &x1, &y1, &x2, &y2 );
        QCOORD corners[] = { x1,y1, x2,y1, x1,y2, x2,y2 };
        p->fillRect( r, color1 );
        p->setPen( color0 );
        p->drawPoints( QPointArray(4, corners) );
        break;
    }

    default:
        KStyle::drawComplexControlMask(control, p, widget, r, opt);
    }
    //KStyle::drawComplexControlMask(control, p, widget, r, opt);
}


int MandrakeStyle::pixelMetric(PixelMetric m, const QWidget *widget) const
{
    //to do use it.
    switch(m)
    {
    case PM_SplitterWidth:
        return 7;
    case PM_ButtonMargin:// Space btw. frame and label
      return /*4*/10;
    case PM_DefaultFrameWidth:
      if ( widget && widget->inherits("QLineEdit"))
	return 2;
      return 1;
    case PM_MenuButtonIndicator:
        return 13;
    case PM_SliderLength:
        return 31;
    case PM_SliderControlThickness:
        return 15;
    case PM_SliderThickness:
        return 15;
    case PM_ExclusiveIndicatorWidth:	// Radiobutton size
        return 13;
    case PM_ExclusiveIndicatorHeight:
        return 13;
    case PM_SpinBoxFrameWidth:
        return 2;
    case PM_CheckListButtonSize: //TODO FIXME !!!!!!!!!!!!!!!
        return 10;
    case PM_ScrollBarExtent:
      return 15; //scrollbar width
    case PM_IndicatorWidth:				// Checkbox size
        return 13;
    case PM_IndicatorHeight:
      return 13;

    default:
        return KStyle::pixelMetric(m, widget);
    }
}


QSize MandrakeStyle::sizeFromContents( ContentsType contents,
                                       const QWidget* widget,
                                       const QSize &contentSize,
                                       const QStyleOption& opt ) const
{
    switch (contents)
    {
    case CT_SpinBox:
        return QSize( contentSize.width(), contentSize.height()+1);
    case CT_ToolButton:
    {
        bool onToolbar = widget->parentWidget() && widget->parentWidget()->inherits( "QToolBar" );
        if (!onToolbar) //Behaves like a button, so scale appropriately to the border
        {
            int w = contentSize.width();
            int h = contentSize.height();
            return QSize( w + 12, h + 10 );
        }
        else
        {
            return KStyle::sizeFromContents( contents, widget, contentSize, opt );
        }
    }
    case CT_ComboBox:
    {
        const QComboBox* combobox = (const QComboBox*) widget;
        //int w  = contentSize.width();
        int h  = contentSize.height();

        if ( h < 26 )
            h = 26;

        int arrow = 11;
        return QSize( contentSize.width() + arrow + (combobox->editable() ? 26 : 22),
                      contentSize.height() + 10 );
    }
    // PUSHBUTTON SIZE
    // ------------------------------------------------------------------
    case CT_PushButton:
    {
        const QPushButton* button = (const QPushButton*) widget;
        int w  = contentSize.width();
        int h  = contentSize.height();
        int bm = pixelMetric( PM_ButtonMargin, widget );
        int fw = pixelMetric( PM_DefaultFrameWidth, widget ) * 2;

        w += bm + fw + 6;	// ### Add 6 to make way for bold font.
        h += bm + fw;

        // Ensure we stick to standard width and heights.
        if ( button->isDefault() || button->autoDefault() )
        {
            if ( w < 80 && !button->pixmap() )
                w = 80;
        }

        if ( h < 30 )
            h = 30;

        return QSize( w, h );
    }
    case CT_PopupMenuItem: {
        if ( ! widget || opt.isDefault() )
            return contentSize;

        const QPopupMenu *popup = (const QPopupMenu *) widget;
        bool checkable = popup->isCheckable();
        QMenuItem *mi = opt.menuItem();
        int maxpmw = opt.maxIconWidth();
        int w = contentSize.width(), h = contentSize.height();

        if ( mi->custom() ) {
            w = mi->custom()->sizeHint().width();
            h = mi->custom()->sizeHint().height();
            if ( ! mi->custom()->fullSpan() )
                h += 2*itemVMargin + 2*itemFrame;
        }
        else if ( mi->widget() ) {
        } else if ( mi->isSeparator() ) {
            w = 30; // Arbitrary
            h = 2;
        }
        else {
            if ( mi->pixmap() )
                h = QMAX( h, mi->pixmap()->height() + 2*itemFrame );
            else {
                // Ensure that the minimum height for text-only menu items
                // is the same as the icon size used by KDE.
                h = QMAX( h, 16 + 2*itemFrame );
                h = QMAX( h, popup->fontMetrics().height()
                          + 2*itemVMargin + 2*itemFrame );
            }

            if ( mi->iconSet() )
                h = QMAX( h, mi->iconSet()->pixmap(
                              QIconSet::Small, QIconSet::Normal).height() +
                          2 * itemFrame );
        }

        if ( ! mi->text().isNull() && mi->text().find('\t') >= 0 )
            w += 12;
        else if ( mi->popup() )
            w += 2 * arrowHMargin;

        if ( maxpmw )
            w += maxpmw + 6;
        if ( checkable && maxpmw < 20 )
            w += 20 - maxpmw;
        if ( checkable || maxpmw > 0 )
            w += 12;

        w += rightBorder;

        return QSize( w, h );
    }

    default:
        return KStyle::sizeFromContents( contents, widget, contentSize, opt );
    }
}


// Fix Qt's wacky image alignment
QPixmap MandrakeStyle::stylePixmap(StylePixmap stylepixmap,
                                   const QWidget* widget,
                                   const QStyleOption& opt) const
{
    return KStyle::stylePixmap(stylepixmap, widget, opt);
}


bool MandrakeStyle::eventFilter( QObject *object, QEvent *event )
{
    //FIXME !!!!!!!!!!!
    //const MandrakeColorData *cdata = lookupData(cg);
    if (KStyle::eventFilter( object, event ))
        return true;
    QToolBar* toolbar;
    //Hover highlight on buttons and combos
    if ( object->inherits("QPushButton") || object->inherits("QComboBox") || object->inherits("QScrollBar") ||  object->inherits("QHeader") )
    {
        if ( (event->type() == QEvent::Enter) && static_cast<QWidget*>(object)->isEnabled())
        {
            QWidget* button = static_cast<QWidget*>(object);
            hoverWidget = button;
            button->repaint( false );
        }
        else if ( (event->type() == QEvent::Leave) && (object == hoverWidget) )
        {
            QWidget* button = static_cast<QWidget*>(object);
            hoverWidget = 0;
            button->repaint( false );
        }
    }
    else if ( object->parent() && !qstrcmp( object->name(), kdeToolbarWidget ) )
    {
        // Draw a gradient background for custom widgets in the toolbar
        // that have specified a "kde toolbar widget" name.

        if (event->type() == QEvent::Paint ) {
            // Find the top-level toolbar of this widget, since it may be nested in other
            // widgets that are on the toolbar.
            QWidget *widget = static_cast<QWidget*>(object);
            QWidget *parent = static_cast<QWidget*>(object->parent());
            int x_offset = widget->x(), y_offset = widget->y();
            while (parent && parent->parent() && !qstrcmp( parent->name(), kdeToolbarWidget ) )
            {
                x_offset += parent->x();
                y_offset += parent->y();
                parent = static_cast<QWidget*>(parent->parent());
            }

            QRect r  = widget->rect();
            QRect pr = parent->rect();
            bool horiz_grad = pr.width() < pr.height();

            // Check if the parent is a QToolbar, and use its orientation, else guess.
            QToolBar* tb = dynamic_cast<QToolBar*>(parent);
            if (tb) horiz_grad = tb->orientation() == Qt::Vertical;

            QPainter p( widget );
            renderGradient(&p, r, parent->colorGroup().button(),parent->colorGroup().button(), horiz_grad,
                           x_offset, y_offset, pr.width(), pr.height());
            return false;	// Now draw the contents
        }
    } else if ( object->parent() &&
                (toolbar = dynamic_cast<QToolBar*>(object->parent())) )
    {

        // We need to override the paint event to draw a
        // gradient on a QToolBarExtensionWidget.
        if ( event->type() == QEvent::Paint ) {
            QWidget *widget = static_cast<QWidget*>(object);
            QRect wr = widget->rect(), tr = toolbar->rect();
            QPainter p( widget );
            renderGradient(&p, wr, toolbar->colorGroup().button(),toolbar->colorGroup().button(),
                           toolbar->orientation() == Qt::Vertical,
                           wr.x(), wr.y(), tr.width() - 2, tr.height() - 2);
            p.setPen( toolbar->colorGroup().dark() );
            if ( toolbar->orientation() == Qt::Horizontal )
                p.drawLine( wr.width()-1, 0, wr.width()-1, wr.height()-1 );
            else
	      p.drawLine( 0, wr.height()-1, wr.width()-1, wr.height()-1 );
            return true;
        }
    }
    return false;
}

#include <iostream>
void MandrakeStyle::renderGradient( QPainter* p,
                                    const QRect& r,
                                    const QColor& ca,
                                    const QColor& cb ,
                                    bool horizontal,
                                    int px,
                                    int py,
                                    int pwidth,
                                    int pheight) const
{
    // px, py specify the gradient pixmap offset relative to the top-left corner.
    // pwidth, pheight specify the width and height of the parent's pixmap.
    // We use these to draw parent-relative pixmaps for toolbar buttons
    // and menubar items.

    QString gradName = QString( "gradient_%1_%2").arg( ca.name()).arg( cb.name());
    GradientSet* grSet = gDict.find( gradName );
    if (!grSet) {
        grSet = new GradientSet(ca, cb);
        gDict.insert( gradName, grSet );
    }

    if (horizontal) {
        int width = (pwidth != -1) ? pwidth : r.width();

        if (width <= 34)
            p->drawTiledPixmap(r, *grSet->gradient(HMed), QPoint(px, 0));
        else if (width <= 52)
            p->drawTiledPixmap(r, *grSet->gradient(HLarge), QPoint(px, 0));
        else {
#if 0
            KPixmap *hLarge = grSet->gradient(HLarge);

            // Don't draw a gradient if we don't need to
            if (hLarge->width() > px)
            {
                int pixmapWidth = hLarge->width() - px;

                // Draw the gradient
                p->drawTiledPixmap( r.x(), r.y(), pixmapWidth, r.height(),
                                    *hLarge, px, 0 );
                // Draw the remaining fill
                p->fillRect(r.x()+pixmapWidth, r.y(), r.width()-pixmapWidth,
                            r.height(),	ca);

            } else
#endif
                p->fillRect(r, ca);
        }
    } else {
        // Vertical gradient
        // -----------------
        int height = (pheight != -1) ? pheight : r.height();

        if (height <= 24)
            p->drawTiledPixmap(r, *grSet->gradient(VSmall), QPoint(0, py));
        else if (height <= 34)
            p->drawTiledPixmap(r, *grSet->gradient(VMed), QPoint(0, py));
        else if (height <= 64)
            p->drawTiledPixmap(r, *grSet->gradient(VLarge), QPoint(0, py));
        else {
#if 0
            KPixmap *vLarge = grSet->gradient(VLarge);

            // Only draw the upper gradient if we need to.
            if (vLarge->height() > py)
            {
                int pixmapHeight = vLarge->height() - py;

                // Draw the gradient
                p->drawTiledPixmap( r.x(), r.y(), r.width(), pixmapHeight,
                                    *vLarge, 0, py );
                // Draw the remaining fill
                p->fillRect(r.x(), r.y()+pixmapHeight, r.width(),
                            r.height()-pixmapHeight, ca);

            } else
#endif
                p->fillRect(r, ca);
        }
    }

}




//copy from qt->drawUtils::qDrawShadeRect
// fix color of qgroupbox
void MandrakeStyle::drawShadeRect( QPainter *p, int x, int y, int w, int h,
		     const QColorGroup &g, bool sunken,
		     int lineWidth, int midLineWidth,
		     const QBrush *fill )const
{
    if ( w == 0 || h == 0 )
	return;
    if ( ! ( w > 0 && h > 0 && lineWidth >= 0 && midLineWidth >= 0 ) ) {
	return;
    }
    QPen oldPen = p->pen();
    if ( sunken )
	p->setPen( /*g.dark()*/g.mid() );
    else
	p->setPen( g.light() );
    int x1=x, y1=y, x2=x+w-1, y2=y+h-1;
    QPointArray a;

    if ( lineWidth == 1 && midLineWidth == 0 ) {// standard shade rectangle
	p->drawRect( x1, y1, w-1, h-1 );
	if ( sunken )
	    p->setPen( g.light() );
	else
	    p->setPen( /*g.dark()*/g.mid() );
	a.setPoints( 8, x1+1,y1+1, x2-2,y1+1, x1+1,y1+2, x1+1,y2-2,
		     x1,y2, x2,y2,  x2,y1, x2,y2-1 );
	p->drawLineSegments( a );		// draw bottom/right lines
    } else {					// more complicated
	int m = lineWidth+midLineWidth;
	int i, j=0, k=m;
	for ( i=0; i<lineWidth; i++ ) {		// draw top shadow
	    a.setPoints( 8, x1+i, y2-i, x1+i, y1+i, x1+i, y1+i, x2-i, y1+i,
			 x1+k, y2-k, x2-k, y2-k, x2-k, y2-k, x2-k, y1+k );
	    p->drawLineSegments( a );
	    k++;
	}
	p->setPen( g.mid() );
	j = lineWidth*2;
	for ( i=0; i<midLineWidth; i++ ) {	// draw lines in the middle
	    p->drawRect( x1+lineWidth+i, y1+lineWidth+i, w-j, h-j );
	    j += 2;
	}
	if ( sunken )
	    p->setPen( g.light() );
	else
	    p->setPen( /*g.dark()*/g.mid() );
	k = m;
	for ( i=0; i<lineWidth; i++ ) {		// draw bottom shadow
	    a.setPoints( 8, x1+1+i,y2-i, x2-i, y2-i, x2-i, y2-i, x2-i, y1+i+1,
			 x1+k, y2-k, x1+k, y1+k, x1+k, y1+k, x2-k, y1+k );
	    p->drawLineSegments( a );
	    k++;
	}
    }
    if ( fill ) {
	QBrush oldBrush = p->brush();
	int tlw = lineWidth + midLineWidth;
	p->setPen( Qt::NoPen );
	p->setBrush( *fill );
	p->drawRect( x+tlw, y+tlw, w-2*tlw, h-2*tlw );
	p->setBrush( oldBrush );
    }
    p->setPen( oldPen );			// restore pen
}

//fix separator
void MandrakeStyle::drawShadeLine( QPainter *p, int x1, int y1, int x2, int y2,
		     const QColorGroup &cg, bool sunken,
		     int lineWidth, int midLineWidth )const
{
    if (!( p && lineWidth >= 0 && midLineWidth >= 0 ) )	{
	return;
    }
    int tlw = lineWidth*2 + midLineWidth;	// total line width
    QPen oldPen = p->pen();			// save pen
    if ( sunken )
	p->setPen( cg.mid()/*g.dark()*/ );
    else
	p->setPen( cg.light() );
    QPointArray a;
    int i;
    if ( y1 == y2 ) {				// horizontal line
	int y = y1 - tlw/2;
	if ( x1 > x2 ) {			// swap x1 and x2
	    int t = x1;
	    x1 = x2;
	    x2 = t;
	}
	x2--;
	for ( i=0; i<lineWidth; i++ ) {		// draw top shadow
	    a.setPoints( 3, x1+i, y+tlw-1-i,
			 x1+i, y+i,
			 x2-i, y+i );
	    p->drawPolyline( a );
	}
	if ( midLineWidth > 0 ) {
	    p->setPen( cg.mid() );
	    for ( i=0; i<midLineWidth; i++ )	// draw lines in the middle
		p->drawLine( x1+lineWidth, y+lineWidth+i,
			     x2-lineWidth, y+lineWidth+i );
	}
	if ( sunken )
	    p->setPen( /*g.light()*/cg.base() );
	else
	    p->setPen( cg.mid()/*g.dark()*/ );
	for ( i=0; i<lineWidth; i++ ) {		// draw bottom shadow
	    a.setPoints( 3, x1+i, y+tlw-i-1,
			 x2-i, y+tlw-i-1,
			 x2-i, y+i+1 );
	    p->drawPolyline( a );
	}
    }
    else if ( x1 == x2 ) {			// vertical line
	int x = x1 - tlw/2;
	if ( y1 > y2 ) {			// swap y1 and y2
	    int t = y1;
	    y1 = y2;
	    y2 = t;
	}
	y2--;
	for ( i=0; i<lineWidth; i++ ) {		// draw left shadow
	    a.setPoints( 3, x+i, y2,
			 x+i, y1+i,
			 x+tlw-1, y1+i );
	    p->drawPolyline( a );
	}
	if ( midLineWidth > 0 ) {
	    p->setPen( cg.mid() );
	    for ( i=0; i<midLineWidth; i++ )	// draw lines in the middle
		p->drawLine( x+lineWidth+i, y1+lineWidth, x+lineWidth+i, y2 );
	}
	if ( sunken )
	    p->setPen( cg.base()/*.light()*/ );
	else
	    p->setPen( cg.dark()/*g.mid()*/ );
	for ( i=0; i<lineWidth; i++ ) {		// draw right shadow
	    a.setPoints( 3, x+lineWidth, y2-i,
			 x+tlw-i-1, y2-i,
			 x+tlw-i-1, y1+lineWidth );
	    p->drawPolyline( a );
	}
    }
    p->setPen( oldPen );
}

//todo fix me
void MandrakeStyle::drawPlainRect( QPainter *p, int x, int y, int w, int h,
                                   const QColor &c,
                                   int lineWidth, const QBrush *fill )const
{
    if ( w == 0 || h == 0 )
	return;
    if ( !( w > 0 && h > 0 && lineWidth >= 0 ) ) {
#if defined(QT_CHECK_RANGE)
	qWarning( "qDrawPlainRect() Invalid parameters." );
#endif
    }
    QPen   oldPen   = p->pen();
    QBrush oldBrush = p->brush();
    p->setPen( c );
    p->setBrush( Qt::NoBrush );
    for ( int i=0; i<lineWidth; i++ )
	p->drawRect( x+i, y+i, w-i*2, h-i*2 );
    if ( fill ) {				// fill with fill color
	p->setPen( Qt::NoPen );
	p->setBrush( *fill );
	p->drawRect( x+lineWidth, y+lineWidth, w-lineWidth*2, h-lineWidth*2 );
    }
    p->setPen( oldPen );
    p->setBrush( oldBrush );
}



void MandrakeStyle::drawButton( const MandrakeColorData *cdata, const QRect & r, QPainter *p,const QColorGroup &cg, SFlags flags)const
{
    QPen oldPen = p->pen();
    bool down = flags & Style_Down;
    bool on   = flags & Style_On;

    bool sunken = on || down;
    int x, y, xw, yh;
    bool mouseOver = flags & Style_MouseOver;
    x= r.x();
    y= r.y();
    xw = r.x()+r.width()-1;
    yh = r.y()+r.height()-1;
    p->fillRect(r,QBrush(cdata->shades[1]));

    if( sunken )
      p->setPen( cg.background().dark(/*112*/150) /*??????????????????*/);
    else
        p->setPen( cdata->shades[4]);
    p->drawPoint( x+3,y+3);
    p->drawLine( x+4,y+2,xw-5,y+2);
    p->drawLine( x+2,y+4,x+2,yh-5);

    p->setPen( cdata->shades[5]);
    p->drawPoint( xw-4,y+3);
    p->drawLine( xw-3,y+4,xw-3,yh-5);
    p->drawPoint( xw-4,yh-4);
    p->drawLine( x+4,yh-3,xw-5,yh-3);
    p->drawPoint( x+3,yh-4);

    if( sunken )
    {
/*white*/
        // Button default indicator
        p->setPen( cg.base());
        p->drawLine( xw -4,y+4,xw-4,yh-5);
        p->drawLine( x+4,yh-4,xw-5,yh-4);

        /*grey 3*/

        p->setPen( cdata->shades[3]);
        p->drawLine( x+4, y+1, xw-6, y+1 );
        p->drawLine( x+1, y+4, x+1, yh-6 );
        p->drawPoint( x+3,y+2);
        p->drawPoint( x+2,y+3);
        /*grey2 */
        p->setPen( cdata->shades[2]);
        p->drawPoint( x+1,yh-5);
        p->drawPoint( xw-5,y+1);
        p->drawPoint( x+3,y+1);
        p->drawPoint( x+2,y+2);
        p->drawPoint( x+1,y+3);

        //p->drawPoint( x+1, yh );
	if( !mouseOver )
	  renderGradient(p, QRect(x+4, y+4, xw-8, yh-8),
			 cdata->shades[2] ,cdata->shades[0], false);

    }
    else if( !sunken && (flags & Style_Enabled ))
    {
        // Button default indicator
        p->setPen( cg.base());
        p->drawLine( x+4, y+3, xw-5, y+3 );
        p->drawLine( x+3, y+4, x+3, yh-5 );

        /*grey 3*/
        p->setPen( cdata->shades[3]);
        p->drawLine( xw-2,y+5,xw-2,yh-5);
        p->drawPoint( xw-3,yh-4);
        p->drawPoint( xw-4,yh-3);
        p->drawLine( x+5,yh-2,xw-5,yh-2);

        p->setPen( cdata->shades[2]);
        p->drawPoint( x+4, yh-2 );
        p->drawPoint( xw-2, y+4 );
	p->drawPoint( xw-2, yh-4);
	p->drawPoint( xw-3, yh-3);
	p->drawPoint( xw-4, yh-2);
	if( !mouseOver)
	  renderGradient(p, QRect(x+4, y+4, xw-8, yh-8),
			 cdata->shades[0] ,cdata->shades[2], false);

    }
    //TODO fixme !!!! bad gradient !
    if ( mouseOver && !sunken )
        renderGradient(p, QRect(x+4, y+4, xw-8, yh-8),
                       cg.base(), cdata->shades[1], false);
    p->setPen( oldPen );			// restore pen

}

QRect MandrakeStyle::subRect(SubRect r, const QWidget *widget) const
{
    	switch ( r )
	{
        case SR_ProgressBarContents:
        {
            const QProgressBar* bar = (const QProgressBar*) widget;
            QRect wrect( widget->rect());
            return QRect( wrect.x() , wrect.y() , wrect.width(),  wrect.height() );
        }
        case SR_PushButtonFocusRect:
        {
            const QPushButton* button = (const QPushButton*) widget;
            QRect wrect(widget->rect());
            if (button->isDefault() || button->autoDefault())
            {
                return QRect(wrect.x() + 6, wrect.y() + 5, wrect.width() - 13,  wrect.height() - 10);
            }
            else
            {
                return QRect(wrect.x() + 4 , wrect.y() + 5, wrect.width() - 9,  wrect.height() - 10);
            }

            break;
        }
        case SR_ComboBoxFocusRect:
        {
            return querySubControlMetrics( CC_ComboBox, widget, SC_ComboBoxEditField );
        }
        default:
            return KStyle::subRect( r, widget );
	}
}


void MandrakeStyle::drawHandlePoint ( const MandrakeColorData *cdata,QPainter *p, int x, int y)const
{
    p->save();
    //galaxy_data->gray_gc[7]
    p->setPen( cdata->shades[7]);
    p->drawPoint(x+1,y);
    p->drawPoint(x+2,y);
    p->drawPoint(x,y+1);
    p->drawPoint(x+1,y+1);

    //galaxy_data->gray_gc[4]
    p->setPen( cdata->shades[4]);
    p->drawPoint(x+2,y+1);
    p->drawPoint(x+3,y+1);
    p->drawPoint(x+1,y+2);

    //galaxy_data->gray_gc[2]
    p->setPen( cdata->shades[2]);
    p->drawPoint(x,y+2);
    p->drawPoint(x+1,y+3);
    p->drawPoint(x+3,y+3);

    //galaxy_data->gray_gc[0]
    p->setPen( cdata->shades[0]);
    p->drawPoint(x+2,y+2);
    p->drawPoint(x+2,y+3);
    p->restore();
}
