/*
 * Copyright (C) 2003 Laurent Montel<lmontel@mandrakesoft.com>
 *
 * based on KDE3 HighColor Style (version 1.0)
 * Copyright (C) 2001-2002 Karol Szwed      <gallium@kde.org>
 *           (C) 2001-2002 Fredrik Höglund  <fredrik@kde.org>
 *
 * Drawing routines adapted from the KDE2 HCStyle,
 * Copyright (C) 2000 Daniel M. Duley       <mosfet@kde.org>
 *           (C) 2000 Dirk Mueller          <mueller@kde.org>
 *           (C) 2001 Martijn Klingens      <klingens@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License version 2 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef __MANDRAKESTYLE_H
#define __MANDRAKESTYLE_H

#include <qbitmap.h>
#include <qintdict.h>
#include <kdrawutil.h>
#include <kpixmap.h>
#include <kstyle.h>
#include <qintcache.h>

class QPopupMenu;


enum GradientType{ VSmall=0, VMed, VLarge, HMed, HLarge, GradientCount };

class GradientSet
{
public:
    GradientSet(const QColor &_begin, const QColor &_end);
    ~GradientSet();

    KPixmap* gradient(GradientType type);
    QColor* beginColor()  { return(&begin); }
    QColor* endColor() { return(&end); }
private:
    KPixmap *gradients[5];
    QColor begin;
    QColor end;

};



class MandrakeStyle : public KStyle
{
    Q_OBJECT

public:
    MandrakeStyle();
    virtual ~MandrakeStyle();

    void polish( QWidget* widget );
    void unPolish( QWidget* widget );

    void renderMenuBlendPixmap( KPixmap& pix, const QColorGroup &cg,
                                const QPopupMenu* popup ) const;

    void drawKStylePrimitive( KStylePrimitive kpe,
                              QPainter* p,
                              const QWidget* widget,
                              const QRect &r,
                              const QColorGroup &cg,
                              SFlags flags = Style_Default,
                              const QStyleOption& = QStyleOption::Default ) const;

    void drawPrimitive( PrimitiveElement pe,
                        QPainter* p,
                        const QRect &r,
                        const QColorGroup &cg,
                        SFlags flags = Style_Default,
                        const QStyleOption& = QStyleOption::Default ) const;

    void drawControl( ControlElement element,
                      QPainter *p,
                      const QWidget *widget,
                      const QRect &r,
                      const QColorGroup &cg,
                      SFlags flags = Style_Default,
                      const QStyleOption& = QStyleOption::Default ) const;

    void drawControlMask( ControlElement element,
                          QPainter *p,
                          const QWidget *widget,
                          const QRect &r,
                          const QStyleOption& = QStyleOption::Default ) const;

    void drawComplexControl( ComplexControl control,
                             QPainter *p,
                             const QWidget *widget,
                             const QRect &r,
                             const QColorGroup &cg,
                             SFlags flags = Style_Default,
                             SCFlags controls = SC_All,
                             SCFlags active = SC_None,
                             const QStyleOption& = QStyleOption::Default ) const;

    void drawComplexControlMask( ComplexControl control,
                                 QPainter *p,
                                 const QWidget *widget,
                                 const QRect &r,
                                 const QStyleOption& = QStyleOption::Default ) const;

    int pixelMetric( PixelMetric m,
                     const QWidget *widget = 0 ) const;

    QSize sizeFromContents( ContentsType contents,
                            const QWidget *widget,
                            const QSize &contentSize,
                            const QStyleOption& opt ) const;

    QRect subRect( SubRect r,
                   const QWidget *widget ) const;

    // Fix Qt3's wacky image positions
    QPixmap stylePixmap( StylePixmap stylepixmap,
                         const QWidget *widget = 0,
                         const QStyleOption& = QStyleOption::Default ) const;

    QRect querySubControlMetrics( ComplexControl control,
                                  const QWidget* widget,
                                  SubControl subcontrol,
                                  const QStyleOption& opt = QStyleOption::Default ) const;

protected:
    bool eventFilter( QObject *object, QEvent *event );

    void renderGradient(QPainter* p,
                        const QRect& r,
                        const QColor& ca,
                        const QColor& cb ,
                        bool horizontal,
                        int px=0,
                        int py=0,
                        int pwidth=-1,
                        int pheight=-1 ) const;

    QWidget    *hoverWidget;

    struct MandrakeColorData {
        QRgb buttonColor;
        QRgb spotColor;

        QColor shades[8];
        //QColor spots[3];
        QColor blueMdkColor[3];
        QPixmap *radioPix[8];
        QPixmap *radioMask;
        QColor radioColor;
        ~MandrakeColorData();

        bool isGroup (const QColorGroup &cg) {
	    return cg.button().rgb() == buttonColor && cg.highlight().rgb() == spotColor;
        }
    };


private:
    // Disable copy constructor and = operator
    MandrakeStyle( const MandrakeStyle & );
    MandrakeStyle& operator=( const MandrakeStyle & );
    void drawShadeRect( QPainter *p, int x, int y, int w, int h,
                        const QColorGroup &g, bool sunken,
                        int lineWidth, int midLineWidth,
                        const QBrush *fill=0 )const;

    void drawShadeLine( QPainter *p, int x1, int y1, int x2, int y2,
                        const QColorGroup &g, bool sunken,
                        int lineWidth, int midLineWidth )const;

    void drawPlainRect( QPainter *p, int x, int y, int w, int h,
                        const QColor &c,
                        int lineWidth, const QBrush *fill=0 )const;

    void drawButton( const MandrakeColorData *cdata, const QRect & r, QPainter *p,const QColorGroup &cg, SFlags flags)const;

    void drawHandlePoint ( const MandrakeColorData *cdata,QPainter *p, int x, int y)const;

    void drawHandle( const MandrakeColorData *cdata, QPainter *p,const QRect &r,const QColorGroup &cg,SFlags flags, const QStyleOption& opt, bool drawBorderAndGradient, bool drawGradient = false ) const;

    void drawFrame( const MandrakeColorData *cdata, const QColorGroup & cg, QPainter *p, const QRect & r, bool checkbox = false )const;
    void drawComboboxArrow( QPainter *p,
                            const QWidget *widget,
                            const QRect &r,
                            const QColorGroup &cg,
                            SFlags flags,
                            SCFlags active)const;
    void drawMandrakeArrow( QPointArray a, QPainter *p,
                            const QRect &r,
                            const QColorGroup &cg,
                            SFlags flags,
                            const QStyleOption& opt ) const;


    MandrakeColorData *realizeData (const QColorGroup &cg) const;
    const MandrakeColorData *lookupData (const QColorGroup &cg) const;

    void getShade (const QColorGroup &cg, int shadenr, QColor &res) const;
    QIntCache<MandrakeColorData> m_dataCache;
    static const double shading[8];
};

#endif
